<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;

class CampanhaPromocional extends \Teknisa\MAN\Controllers\TeknisaCrud {
    const CAMPANHA = 'vnd48000_campromo';
    const PROD = 'vnd48000_campromo_combinacao_prod';
    const CAMPFILI = 'vnd48000_campromo_filiaisparticipantes';
    
    protected $dataSourceName = CampanhaPromocional::CAMPANHA;
    protected $parameterBagColumns =  array('NRORG', 'CDCAMPPROMO');
    protected $util;
    
    public function __construct($crudService, Utilities\Util $util) {
        $this->crudService = $crudService;
        $this->util = $util;
    }
    
    public function find (DTO\Request\Filter $request, DTO\Response $response)  {
        $conditions = $this->util->getParamsAsArray($request);
        if(!isset($conditions['CDCAMPPROMO'])){
            $request->getFilterCriteria()->addCondition('CDCAMPPROMO','=', 'T');
        }
        parent::find($request, $response);
    }
    
    public function findProd (DTO\Request\Filter $request, DTO\Response $response)  {
        $this->dataSourceName = CampanhaPromocional::PROD;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPPROMO');
        
        parent::find($request, $response);
        
        $this->parameterBagColumns =  array('NRORG');
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;
    }  
    
    
    public function findCampFilial (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = CampanhaPromocional::CAMPFILI;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPPROMO');

        parent::find($request, $response);
        
        $this->parameterBagColumns = null;
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;
    }
    
    public function saveFilialParticipantes (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = CampanhaPromocional::CAMPFILI;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPPROMO');
        
        $rows = $request->getDataSet()->getRows();
        $filiais = $rows[0]['CDFILIAL'];
        
        for($i = 0; $i < count($filiais); $i++) {
            $rows[0]['CDFILIAL'] = $filiais[$i];
            parent::save($request, $response);
        }
        
        $this->parameterBagColumns = null;
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;    
    }
    
    public function saveCombinaProduto (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = CampanhaPromocional::PROD;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPPROMO');
        parent::save($request, $response);
        
        $this->parameterBagColumns = null;
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;    
    }
    
    public function removeFilialParticipantes (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = CampanhaPromocional::CAMPFILI;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPPROMO','CDFILIAL');
        
        parent::delete($request, $response);
        
        $this->parameterBagColumns = null;
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;    
    }
    
    public function removeCombinaProduto (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = CampanhaPromocional::PROD;
        $this->parameterBagColumns = array('NRORG', 'CDCAMPPROMO');
        
        parent::delete($request, $response);
        
        $this->parameterBagColumns = null;
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;    
	}
	
	public function deleteCampanha(DTO\Request\Row $request, DTO\Response $response) {
	    $this->dataSourceName = CampanhaPromocional::CAMPANHA;
        $this->parameterBagColumns = array('CDCAMPPROMO', 'NRORG');
        
        parent::delete($request, $response);
        
        $this->parameterBagColumns = null;
        $this->dataSourceName = CampanhaPromocional::CAMPANHA;    
	}
	
	
}