<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services\FamiliadeProdutoService;
use Teknisa\MAN\Services\NovoCodigo;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\MAN\Utilities;

class FamiliadeProdutos extends \Teknisa\MAN\Controllers\TeknisaCrud  {

    const FAMILIA = 'vnd09300_familiadeprodutos';
    const FAMILIATOFILIAL = 'vnd09300_familiadeprodutos_to_filial';
    const FAMILIAFILI = 'vnd09300_familiadeprodutos_filial';
    const FAMILIAFILIPROD = 'vnd09300_familiadeprodutos_filial_produto';
    const PRODUCTTOSAVE = 'vnd09300_familiadeprodutos_produtos';
    
    protected $dataSourceName;
    protected $parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDFAMILISALD', 'CDNVPRODUTO');
    protected $FamiliadeProdutoService;
    protected $novoCodigo;
    
    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, $crudService, FamiliadeProdutoService $FamiliadeProdutoService, NovoCodigo $novoCodigo, Utilities\Util $utilService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->dataSourceManager       = $dataSourceManager;
        $this->crudService             = $crudService;
        $this->FamiliadeProdutoService = $FamiliadeProdutoService;
        $this->novoCodigo              = $novoCodigo;
        $this->utilService             = $utilService;
    }
    
    public function findProductToFamiliaToFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::PRODUCTTOSAVE;
        parent::find($request, $response);
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIA;
        parent::find($request, $response);
    }
    
    public function findFamiliaToFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIATOFILIAL;
        parent::find($request, $response);
    }
    
    public function saveFamilia(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIA;
        parent::save($request, $response);
    }
    
    public function deleteFamilia(DTO\Request\Row $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIA;
        self::deleteRowWithoutNotification($request, $response);
    }
    
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIAFILI;
        parent::find($request, $response);
    }
    
    public function saveFamiliaFilial(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIAFILI;
        parent::save($request, $response);
    }
    
    public function deleteFilial(DTO\Request\Row $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIAFILI;
        self::deleteRowWithoutNotification($request, $response);
    }
    
    public function findProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIAFILIPROD;
        parent::find($request, $response);
    }
    
    public function deleteProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIAFILIPROD;
        self::deleteDataSetWithoutNotification($request, $response);
    }
    
    public function saveProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this::FAMILIAFILIPROD;
        $conditions = $request->getDataSet()->getRows()[0];
        $to_valid = $this->FamiliadeProdutoService->ValidProduct($conditions);
        if (empty($to_valid) || !($conditions['__is_new'])){
            parent::save($request, $response);
        }
        else{
            $response->addMessage(new Message('Não é possível inserir. O produto selecionado já está incluído na família: '.$to_valid[0]['CDFAMILISALD'].' - '.$to_valid[0]['NMFAMILISALD']));
            $response->setError(new DTO\Response\Error('Não é possível inserir. O produto selecionado já está incluído na família: '.$to_valid[0]['CDFAMILISALD'].' - '.$to_valid[0]['NMFAMILISALD'], $conditions));
        }
    }
    
    public function deleteRowWithoutNotification(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteDataSetWithoutNotification(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function cadAutomaticoProd(DTO\Request $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        if(!isset($conditions['QTCONSDIAFAM'])){
            $conditions['QTCONSDIAFAM'] = NULL;
        }
        $produtos = $this->FamiliadeProdutoService->selectProductAutoCad($conditions);        
        foreach($produtos as $produto){
            $this->FamiliadeProdutoService->insertProducts($produto);
        }
    }
}