<?php 
namespace Teknisa\MAN\Controllers;
    
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\CrudWithParameterBag;

use Doctrine\ORM\EntityManager;

class GeraNotaVenda extends CrudWithParameterBag { 
    
    protected $dataSourceNameArray = array (
        'FindGeraNotaFiscal' => 'dfs15300_geraNotaVenda',
        'FindSerieNotafiscal' => 'select_serieNotafiscal',
        'FindVendaNotaFiscal' => 'select_vendaNotaFiscal'
    );
    protected $parameterBagColumnsArray = array (
        'FindGeraNotaFiscal' =>  array('CDFILIAL', 'IDSTATUS', 'IDAVISOS', 'ENTRINI','ENTRFIM', 'CDFILIRECE'),
        'FindSerieNotafiscal' =>  array('CDEMPRESA', 'CDINSCESTA', 'NRORG')
    );
    
   public function __construct ($parameterBag, Manager $dataSourceManager, Services\GeraNotaVendaService $GeraNotaVendaService, $utilService) {
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->utilService       = $utilService;
        $this->GeraNotaVendaService = $GeraNotaVendaService;
    }
    
    ///find's
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['FindGeraNotaFiscal'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['FindGeraNotaFiscal'];
        $conditions                = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter          = self::prepareParameters($conditions);
        $requestDataSourceFilter    = $request->getFilterCriteria()->getConditions();
        
        foreach($requestDataSourceFilter as $filter) {
            if($filter['operator'] === 'MAPPED_LIKE_ALL') {
                array_push($dataSourceFilter, $filter);
            }
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        parent::find($newRequest, $response);
    }
    
    public function findSerieNota(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['FindSerieNotafiscal'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['FindSerieNotafiscal'];
        parent::find($request, $response);
    }
    
    public function findVendaNotaFiscal(DTO\Request\Filter $request, DTO\Response $response) {
        
        ini_set('memory_limit', '1G');
        
        $this->dataSourceName = $this->dataSourceNameArray['FindVendaNotaFiscal'];
        $conditions = $this->utilService->getParamsAsArray($request);
        $result = $this->GeraNotaVendaService->getVendas($conditions);
        
        return $response->addDataSet(new DataSet("select_vendaNotaFiscal",  $result));
    }
    
    public function geraNota(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['FindGeraNotaFiscal'];
        $conditions           = $this->utilService->getParamsAsArray($request);
        $result = $this->GeraNotaVendaService->prepareProcedure($conditions[0]);
        return $response->addDataSet(new DataSet("dfs15300_geraNotaVenda",  []));
    }
    
    private function prepareParameters($conditions) {
        if(isset($conditions['DTENTRSAID'])) {
            $ENTRINI = $conditions['DTENTRSAID'][0];
            $ENTRFIM = $conditions['DTENTRSAID'][1];
        } else {
            $ENTRINI = $ENTRFIM = NULL;
        }
        if(isset($conditions['CDFILIAL'])){
            $CDFILIAL = $conditions['CDFILIAL'];
        } else {
            $CDFILIAL = NULL;
        }
        if(isset($conditions['IDSTATUS'])){
            $IDSTATUS = $conditions['IDSTATUS'];
        } else {
            $IDSTATUS = 'T';
        }
        if(isset($conditions['IDAVISOS'])){
            $IDAVISOS = $conditions['IDAVISOS'];
        } else {
            $IDAVISOS = 'T';
        }
        
        $parameters = array(
            'CDFILIAL'  => $CDFILIAL,
            'IDSTATUS'   => $IDSTATUS,
            'IDAVISOS'  => $IDAVISOS,
            'ENTRINI'   => $ENTRINI,
            'ENTRFIM'   => $ENTRFIM
        );
        
        $dataSourceFilter = array();
        
        $dataSourceFilter[0] = array(
            "columnName" => 'CDFILIAL',
            "operator"   => "=",
            "value"      => $parameters['CDFILIAL']
        ); 
        $dataSourceFilter[1] = array(
            "columnName" => 'ENTRINI',
            "operator"   => "=",
            "value"      => $parameters['ENTRINI']
        );
        $dataSourceFilter[2] = array(
            "columnName" => 'IDSTATUS',
            "operator"   => "=",
            "value"      => $parameters['IDSTATUS']
        );
        $dataSourceFilter[3] = array(
            "columnName" => 'IDAVISOS',
            "operator"   => "=",
            "value"      => $parameters['IDAVISOS']
        ); 
        $dataSourceFilter[4] = array(
            "columnName" => 'ENTRFIM',
            "operator"   => "=",
            "value"      => $parameters['ENTRFIM']
        );
        $dataSourceFilter[5] = array(
            "columnName" => 'CDFILIRECE',
            "operator"   => "=",
            "value"      => 'T'
        );
        
        return $dataSourceFilter;
    }
    
   
}
?>