<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;

class SugestaoVenda extends CrudWithParameterBag{
    
    const TABLE = 'SUGESTAOVENDA';
    const CAMPO_PK = 'IDSUGESTAOVENDA';
    
    
    protected $dataSourceNameArray = array (
        'filterSugestao'               => 'confTelaSugestaoVenda',
        'filterProd'                   => 'confTelaSugestaoProd'
    );
    protected $parameterBagColumnsArray = array (
        'filterSugestao'               =>  array("NRORG","CDFILIAL", "NRCONFTELA"),
        'filterProd'                =>  array("NRORG","CDFILIAL", "NRCONFTELA")
    );
    
    protected $novoCodigo;
    protected $SugestaoVendaService;
    protected $dataSourceManager;
    protected $parameterBag;
    protected $util;
     
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, $novoCodigo, $util, $SugestaoVendaService) {
        $this->novoCodigo  = $novoCodigo;
        $this->util = $util;
        parent::__construct($dataSourceManager, $parameterBag);
        $this->SugestaoVendaService = $SugestaoVendaService;
    }
    
    public function save (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['filterSugestao'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterSugestao'];
        $this->setCodigo($request, $request->getDataSet()->getRows());
        parent::save($request, $response);
    }
    
    public function deleteSugestao (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['filterSugestao'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterSugestao'];
        parent::delete($request,$response);
    }
    
    private function setCodigo ($request, $rows){
        foreach ($rows as $key => $row) {
            if($row['__is_new']){
                $novoCodigo = $this->novoCodigo->getCodeByTableName(self::TABLE,$row['NRORG'],1,null,false);
                $request->getDataSet()->getRows()[$key]->set(self::CAMPO_PK, $novoCodigo);
            }
        }
        return $request;
    }
    
     
    
    
    public function findSugestao (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterSugestao'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterSugestao'];
        parent::find($request, $response);
    }
    
    public function findProd (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProd'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProd'];
        parent::find($request, $response);
    }
    
    public function exportSugestaoItem (DTO\Request\DataSet $request, DTO\Response $response){
        $row = $this->util->getParamsAsArray($request)[0];
        $NEWIDSUGESTAOVENDA = $row['NEWIDSUGESTAOVENDA'];
        $row = $this->buildParams($row);
        $CDFILIALORIGEM = $row['CDFILIAL'];
        $NRCONFTELAORIGEM = $row['NRCONFTELA'];
        $IDSUGESTAOVENDAORIGEM = $row['IDSUGESTAOVENDA'];
        $paramsBusca = array(':NRORG' => $row['NRORG'],
                             ':IDSUGESTAOVENDA' => $row['IDSUGESTAOVENDA'],
                             ':CDSUGESTAOITEM' => $row['CDSUGESTAOITEM'],
                             ':TCDSUGESTAOITEM' => $row['TCDSUGESTAOITEM']);
                             
        $itens = $this->SugestaoVendaService->buscaItens($paramsBusca);
        
       

        foreach($NEWIDSUGESTAOVENDA as $each) {
            $param = array(':IDSUGESTAOVENDA' => $each);
            $this->SugestaoVendaService->deleteItensToExport($param);  
            foreach($itens as $eachItem) {
                $IDSUGESTAOITEM = $this->novoCodigo->getCodeByTableName('SUGESTAOITEM', $eachItem['NRORG'],1,null,false);
                $params = array(':IDSUGESTAOVENDA' => $each,
                                ':IDSUGESTAOITEM' => $IDSUGESTAOITEM,
                                ':IDSUGESTAOTIPO' => $eachItem['IDSUGESTAOTIPO'] ,
                                ':CDSUGESTAOITEM' => $eachItem['CDSUGESTAOITEM'] ,
                                ':NRORDEMSUG' => $eachItem['NRORDEMSUG'],
                                ':NRORG' => $eachItem['NRORG']);
                                
                $this->SugestaoVendaService->saveExportItem($params);
            }
        }
    }
    
    public function buildParams($conditions){
        //$conditions['NEWIDSUGESTAOVENDA'] = implode('_', $conditions['NEWIDSUGESTAOVENDA']);
        
        if (isset($conditions['CDSUGESTAOITEM'])){
            $conditions['CDSUGESTAOITEM'] = implode('_', $conditions['CDSUGESTAOITEM']);
            $conditions['TCDSUGESTAOITEM'] = null;
        } 
        else{
            $conditions['TCDSUGESTAOITEM'] = 'T';
            $conditions['CDSUGESTAOITEM'] = null;
        }
     
        return $conditions;
    }
}