<?php
namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services\TransmissaoNfceService;
use Teknisa\MAN\Controllers\TransmissaoNfceController;
use \Datetime;
use \SimpleXMLElement;

class TransmissaoNFCE {

    private $transmissaoService, $xml;
    private $todasNotas, $notasValidas, $notasInvalidas;
    
    public function __construct(TransmissaoNfceService $transmissaoService, $nota, NotaFiscal $nfce, $instanceManager, $utilService) {
        $this->transmissaoService = $transmissaoService;
        $this->todasNotas = $nota;
        $this->notasInvalidas = $this->notasValidas = array();
        $this->instanceManager = $instanceManager;
        $this->verificarNotasBloqueadas();
        $this->notaFiscal = $nfce;
        $this->utilService = $utilService;
    }
    
    public function prepararXmlNotas() {
        
        TransmissaoNfceController::escreverLog(str_pad('-', 100, "-", STR_PAD_LEFT));
        TransmissaoNfceController::escreverLog("Preparando XML");
        TransmissaoNfceController::escreverLog("");
        
        $time_start = microtime(true);
        $produtosComErro = 0;

        foreach($this->notasValidas as &$nota) {
            if($nota['STATUSCONSULTA'] == '0'){
                TransmissaoNfceController::escreverLog("Nota Fiscal: " . $nota['NRNOTAFISCALCE']);
                $fechamentoTag = strpos($nota['DSARQXMLNFCE'], '</NFe>'); // (pos(cdsNotaFiscalDSARQXMLNFCE.Value,'</NFe>')=0);
                $IDXMLVALIDNFCE = $nota['IDXMLVALIDNFCE']; //(cdsNotaFiscalIDSTATUSNFCE.AsString='R') or  or (cdsNotaFiscalIDXMLVALIDNFCE.AsString='N') or EmptyString(cdsNotaFiscalDSARQXMLNFCE.Value)
                
                /* Foi retirada a condição de verificação do XML !$fechamentoTag or $IDXMLVALIDNFCE == 'R' or $IDXMLVALIDNFCE == 'N' or empty($nota['DSARQXMLNFCE']) */
                
                $SGESTADO = $nota['SGESTADO'] == 'SP' ? '1' : '9'; // iif(cdsNotaFiscalSGESTADO.AsString='SP','1','9');
                $result = $this->regerarXmlNFCe($nota, $SGESTADO);
                if (!$result['error']) {
                    $this->atualizarXMLBanco($this->xml->asXML(), $nota);
                    TransmissaoNfceController::escreverLog("XML OK.");
                } else {
                    $produtosComErro++;
                }
            
                TransmissaoNfceController::escreverLog("");
            }
        }
        
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
        
        if ($produtosComErro > 0) {
            throw new \Exception('Não foi possível gerar o XML de '.$produtosComErro.' produtos. Verifique o LOG para mais informações.');
        }
    }
    
    public function transmitirNotas() {
        TransmissaoNfceController::escreverLog(str_pad('-', 100, "-", STR_PAD_LEFT));
        TransmissaoNfceController::escreverLog("Transmitindo notas");
        TransmissaoNfceController::escreverLog("");
        $time_start = microtime(true);
        
        foreach($this->notasValidas as $nota) {
            if($nota['STATUSCONSULTA'] == '0'){
                TransmissaoNfceController::escreverLog("Nota Fiscal: ".$nota['NRNOTAFISCALCE']);
                $this->transmitirXML($nota);
                TransmissaoNfceController::escreverLog("");
            }
        }
        
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
    }
    
    private function transmitirXML($nfceData) {
        $parametros = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'CDLOJA', 'NRSEQVENDA'], $nfceData);
        $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQTRANSMISSAONFCE, $parametros);
        if(!empty($resposta->getResult())){
            $resposta = $resposta->getResult();
    		/*$nrorg = $nfceData['NRORG'];
    		$cdcaixa = $nfceData['CDCAIXA'];
    		$cdfilial = $nfceData['CDFILIAL'];
    		$dtabercaix = $nfceData['DTABERTUR'];
    		$cdloja = $nfceData['CDLOJA'];
    		$cdoperador = $nfceData['CDOPERADOR'];
    		$idtpemisvend = $nfceData['IDTPEMISVEND'];
    		$nrseqvenda = $nfceData['NRSEQVENDA']; 
    		$VRTOTTRIBIBPT = $resposta['VRTOTTRIBIBPT'];
    		$TOTALVENDA = $nfceData['VRTOTALVENDA'];
    		$NRNOTAFISCALCE = $nfceData['NRNOTAFISCALCE'];
    		$DTEMISSAOVENDA = $nfceData['DTEMISSAONFCE'];
    		$DTEMNFCE = $resposta['DTEMNFCE'];
    		$NRACESSONFCE = $nfceData['NRACESSONFCE'];
    		$simulaImpressao = true;
    		$nrlanctonfce = $nfceData['NRLANCTONFCE'];
    		
    		$this->notaFiscal->transmitirNFCe($nrorg, $cdcaixa, $cdfilial, $dtabercaix, $cdloja, 
    									$cdoperador, $idtpemisvend, $nrseqvenda, $VRTOTTRIBIBPT, 
    									$TOTALVENDA, $NRNOTAFISCALCE, $simulaImpressao, $DTEMNFCE, $NRACESSONFCE, $nrlanctonfce);*/
    		
    		$resposta[0]['CDOPERADOR'] = $nfceData['CDOPERADOR'];
    		$resposta[0]['TOTALVENDA'] = $nfceData['VRTOTALVENDA'];
    		$resposta[0]['DTABERTUR'] = $nfceData['DTABERTUR'];
    		
    		$this->notaFiscal->transmitirNFCeNew($resposta);						
    									
        }else{
            TransmissaoNfceController::escreverLog("Não foi encontrado dados da nota para transmissão!");
        }
        
	}

    public function getNotas() {
        return $this->todasNotas;
    }
    
    public function getQtdNotasValidas() {
        return sizeof($this->notasValidas);
    }
    
    public function verificarNotasBloqueadas() {
        
        $notasValidas = array();
        $notasInvalidas = array();
        
        foreach($this->todasNotas as $nota) {
            if ($nota['IDSTATUSNFCE'] == 'T' || $nota['IDSTATUSNFCE'] == 'A' || $nota['IDSTATUSNFCE'] == 'D' || $nota['NRENVIOSEFAZ'] > 9 ) {
                $notasInvalidas[] = $nota;
            } else {
                $notasValidas[] = $nota;
            }
        }
        
        $this->notasValidas = $notasValidas;
        $this->notasInvalidas = $notasInvalidas;
        
        if (sizeof($this->notasValidas) == 0) {
            throw new \Exception("Não foi selecionada nenhuma Nota Fiscal válida. Notas selecionadas podem estar nas seguintes situações: Transmitida, Aceita, Denegada ou com 9 ou mais envios a SEFAZ.", 3);
        }
        
        return $notasInvalidas;
    }
    
    public function verificarSituacaoNFCeSefaz(){
        $log = array();
        $time_start = microtime(true);
        
        foreach($this->notasValidas as $key => $nota) {
            //cria um parametro no qual sera o que ira validar se precisa transmitir novamente. Default 0 = transmite
            //verifica situacao atual da nota com a SEFAZ
            //condicao que verifica se ha XML formado, caso nao tenha faz o processo de transmissao de forma normal
            if(!empty($nota['DSARQXMLNFCE'])){
                $this->notasValidas[$key]['STATUSCONSULTA'] = $this->atualizaSituacaoNFCe($nota);    
            }else{
                $this->notasValidas[$key]['STATUSCONSULTA'] = 0;
            }
        }
        
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
    }
    
    public function updateChaveAcesso() {
        $log = array();
        
        TransmissaoNfceController::escreverLog(str_pad('-', 100, "-", STR_PAD_LEFT));
        TransmissaoNfceController::escreverLog("Atualizando chave de acesso");
        
        $time_start = microtime(true);

        foreach($this->notasValidas as &$nota) {
            if($nota['STATUSCONSULTA'] == '0'){
                $this->verificarDiferencaNrAcessoNfce($nota);
                $this->verificarPeculiaridadesEstados($nota);    
            }
        }
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
        
    }
    
    public function verificarDiferencaNrAcessoNfce (&$nota) {
        $NRACESSONFCE = substr($nota['NRACESSONFCE'], 35, -1); //copy(cdsNotaFiscalNRACESSONFCE.AsString,36,8);
        $NRLANCTONFCE = substr($nota['NRLANCTONFCE'], 2, 9); //copy(cdsNotaFiscalNRLANCTONFCE.AsString,3,8);

        if ($NRACESSONFCE != $NRLANCTONFCE) {
            $NRACESSONFCE1 = substr($nota['NRACESSONFCE'], 0, 33);
            $NRACESSONFCE2 = substr($nota['NRACESSONFCE'], 34, 9);
            
            $CHAVE = $NRACESSONFCE1.'9'.$NRACESSONFCE2;
            $DIGITO = $this->notaFiscal->calculaDigitoVerificadorNFe($CHAVE); //$this->modulo11($NRACESSONFCE1.'9'.$NRACESSONFCE2);
            $modulo = $CHAVE.$DIGITO;
            
            $parametros = array(
                'NRACESSONFCE' => $modulo,
                'CDFILIAL'     => $nota['CDFILIAL'],
                'CDCAIXA'      => $nota['CDCAIXA'],
                'NRSEQVENDA'   => $nota['NRSEQVENDA']
            );
            
            $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDCHAVEACESSO, $parametros);
            
            $nota['NRACESSONFCE'] = $modulo;
        }
    }
    
    public function verificarPeculiaridadesEstados(&$nota) {
        
        $NRACESSONFCE = $nota['NRACESSONFCE'];
        
        $parametros = array(
            'NRACESSONFCE' => $nota['NRACESSONFCE'],
            'CDFILIAL'     => $nota['CDFILIAL'],
            'CDCAIXA'      => $nota['CDCAIXA'],
            'NRSEQVENDA'   => $nota['NRSEQVENDA']
        );
        
        if ($nota['SGESTADO'] == 'SP' or $nota['SGESTADO'] =='CE') {

            $NRACESSONFCE1 = substr($NRACESSONFCE, 0, 1); // copy(cdsNotaFiscalNRACESSONFCE.AsString,1,2);
            $NRACESSONFCE2 = substr($NRACESSONFCE, 6, 35); // copy(cdsNotaFiscalNRACESSONFCE.AsString,7,28);
            $NRACESSONFCE3 = substr($NRACESSONFCE, 35, 42); // copy(cdsNotaFiscalNRACESSONFCE.AsString,36,8));
            $ANO = date('y'); // Copy(DateToStr(now),9,2);
            $MES = date('m'); // Copy(DateToStr(now),4,2);
            
            $modulo = $this->modulo11($NRACESSONFCE1.$ANO.$MES.$NRACESSONFCE2.'1'.$NRACESSONFCE3);
            $parametros['NRACESSONFCE'] = $modulo;
             
            $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDCHAVEACESSOSP, $parametros);
            
       } else if (substr($nota['NRACESSONFCE'], 33, 33) =='1') {
           $NRACESSONFCE1 = substr($NRACESSONFCE, 0, 33); // copy(cdsNotaFiscalNRACESSONFCE.AsString,1,34);
           $NRACESSONFCE2 = substr($NRACESSONFCE, 34, 9); // copy(cdsNotaFiscalNRACESSONFCE.AsString,36,8));
           $modulo = $NRACESSONFCE1.'9'.$NRACESSONFCE2;
           $parametros['NRACESSONFCE'] = $modulo;
           
           $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDCHAVEACESSO, $parametros);
           
        }
        
        $nota['NRACESSONFCE'] = $parametros['NRACESSONFCE'];
        
    }
    
    public function regerarXmlNFCe(&$nota, $SGESTADO) {
        $parametros = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'CDLOJA', 'NRSEQVENDA'], $nota);
        
        $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQTRANSMISSAONFCE, $parametros);
        $result = array(
            'error'    => false
        );
        
        if ($resposta->error() || empty($resposta->getResult())) {
            throw new \Exception("Erro ao regerar XML. Função: regerarXmlNFCe.".$resposta->getMessage(), 1);
        } else {
            TransmissaoNfceController::escreverLog("Regerando XML");
            $data = $resposta->getResult()[0];
            
            $xml = simplexml_load_string(self::DEFAULT_XML);
            
            $infNFe = $xml->addChild('infNFe');
            
            $this->xmlconfigINFNFE($infNFe, $data);
            $this->xmlconfigINFNFEIDE($infNFe, $data);
            $this->xmlconfigINFNFEEMIT($infNFe, $data);
            $this->xmlconfigINFNFDEST($infNFe, $data);
            $resultDET = $this->xmlconfigDET($infNFe, $resposta->getResult());
            $result['error'] = $resultDET['error'];
            $this->xmlconfigTOTAL($infNFe, $data);
            $this->xmlconfigPAG($infNFe, $data);
            $this->xmlconfigINFADIC($infNFe, $data);
            $this->xml = $xml;
        }
        
        return $result;
        
    }
    
    private function gerarParametros(array $parametros, $objeto) {
        
        $parametrosQuery = array();
        
        foreach ($parametros as $key) {
            if (empty($objeto[$key])) {
                $parametrosQuery[$key] = null;
            } else {
                if (is_array($objeto[$key]))
                    $parametrosQuery[':'.$key] = implode("_", $objeto[$key]);
                else 
                    $parametrosQuery[':'.$key] = $objeto[$key];
            }
        }
        
        return $parametrosQuery;
    }
    
    private function xmlconfigINFNFE($xml, $nota) {
        $xml->addAttribute('Id', 'NFe'.$nota['NRACESSONFCE']);
        $xml->addAttribute('versao', $nota['CDVERSAOXMLNFCE']);
    }
    
    private function xmlconfigINFNFEIDE($xml, $nota) {
        $ide = $xml->addChild('ide');    
        $ide->addChild('cUF', $nota['CDESTADOIBGE']); // cdsVendaNFCe.FieldByName('CDESTADOIBGE').AsString;
        $ide->addChild('cNF', substr($nota['NRLANCTONFCE'], 2, 9)); // copy(cdsVendaNFCe.FieldByName('NRLANCTONFCE').AsString,3,8);
        $ide->addChild('natOp', 'Descricao da Natureza da Operacao: Venda');
        if ($nota['CDVERSAOXMLNFCE'] == '3.10') // cdsVendaNFCe.FieldByName('CDVERSAOXMLNFCE').AsString='3.10';
            $ide->addChild('indPag', '2');
        $ide->addChild('mod', '65'); //55-NFe e 65-NFCe
        $ide->addChild('serie', ltrim($nota['CDSERIENFCE'], '0')); // RemoveZerosEsqStr(cdsVendaNFCe.FieldByName('CDSERIENFCE'   ).AsString);
        $ide->addChild('nNF', ltrim($nota['NRNOTAFISCALCE'], '0')); // RemoveZerosEsqStr(cdsVendaNFCe.FieldByName('NRNOTAFISCALCE').AsString);
        $date = DateTime::createFromFormat('d/m/Y H:i:s', empty($nota['DTEMISSAONFCE']) ? $nota['DTVENDA'] : $nota['DTEMISSAONFCE']);
        $dataFormatada1 = $date->format('Y-m-d');
        $dataFormatada2 = $date->format('H:i:s');
        $dataFormatada = $dataFormatada1 . "T" . $dataFormatada2;
        $ide->addChild('dhEmi', $dataFormatada.$nota['HRDIFUTCFUSO']); //AddChild('dhEmi'   ).NodeValue := FormatDateTime('yyyy-mm-dd"T"hh:nn:ss', cdsVendaNFCe.FieldByName('DTEMISSAONFCE').AsDateTime)+cdsVendaNFCe.FieldByName('HRDIFUTCFUSO').AsString;
        $ide->addChild('tpNF', '1'); //1-Saída
        $ide->addChild('idDest', '1'); //1-Interna
        $ide->addChild('cMunFG', strlen($nota['CDMUNICIBGE']) == 7 ? $nota['CDMUNICIBGE'] : $nota['CDESTADOIBGE'].$nota['CDMUNICIBGE']); // iif(Length(cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString)=7, cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString, cdsVendaNFCe.FieldByName('CDESTADOIBGE').AsString+cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString);               
        $ide->addChild('tpImp', '4'); // '4'; //4-DANFe NFCe
        $stTpEmis = $nota['SGESTADO'] == 'SP' ? '1' : '9';
        $ide->addChild('tpEmis', $stTpEmis); // iif(cdsNotaFiscalSGESTADO.AsString='SP','1','9')
        $ide->addChild('cDV', substr($nota['NRACESSONFCE'], -1)); // copy(cdsVendaNFCe.FieldByName('NRACESSONFCE').AsString,44,44);
        $ide->addChild('tpAmb', $nota['IDTPAMBNFCE']); // cdsVendaNFCe.FieldByName('IDTPAMBNFCE'   ).AsString;
        $ide->addChild('finNFe', '1'); // '1'; //1-NFe Normal
        $ide->addChild('indFinal', '1'); // '1'; //1-Consumidor Final
        $ide->addChild('indPres', '1'); // '1'; //Indicador de presença do comprador no estabelecimento comercial no momento da oepração (0-Não se aplica (ex.: Nota Fiscal complementar ou de ajuste;1-Operação presencial;2-Não presencial, internet;3-Não presencial, teleatendimento;4-NFC-e entrega em domicílio;5-Operação presencial, fora do estabelecimento;9-Não presencial, outros)
        $ide->addChild('procEmi', '0'); // '0'; //0-emissão de NF-e com aplicativo do contribuinte;
        $ide->addChild('verProc', 'Retail Management'); // 'VND20000';
        
       if ($stTpEmis == '9') {
          $ide->addChild('dhCont', $dataFormatada.$nota['HRDIFUTCFUSO']); // FormatDateTime('yyyy-mm-dd"T"hh:nn:ss', cdsVendaNFCe.FieldByName('DTEMISSAONFCE').AsDateTime)+cdsVendaNFCe.FieldByName('HRDIFUTCFUSO').AsString;//Copy(getDataEmissNFCe,20,6);
          $ide->addChild('xJust', 'Motivo da Emissao em Contigencia: Caixa OffLine');
       }
    }
    
    private function xmlconfigINFNFEEMIT($xml, $nota) {
        $emit = $xml->addChild('emit');
        
        $emit->addChild('CNPJ', $nota['NRINSJURFILI']); // SoNumerosStr(cdsVendaNFCe.FieldByName('NRINSJURFILI').AsString);
        $emit->addChild('xNome', $nota['NMRAZSOCFILI']); // cdsVendaNFCe.FieldByName('NMRAZSOCFILI').AsString;
        $emit->addChild('xFant', $nota['NMFILIAL']); // cdsVendaNFCe.FieldByName('NMFILIAL'    ).AsString;
        $enderEmit = $emit->addChild('enderEmit');
          $enderEmit->addChild('xLgr', $nota['DSENDEIEST']); // cdsVendaNFCe.FieldByName('DSENDEIEST' ).AsString;
          $enderEmit->addChild('nro', str_pad('', 3, "0", STR_PAD_LEFT)); // Ajusta('000',3);
          $enderEmit->addChild('xBairro', $nota['NMBAIRIEST']); // cdsVendaNFCe.FieldByName('NMBAIRIEST' ).AsString;
          $enderEmit->addChild('cMun', strlen($nota['CDMUNICIBGE']) == 7 ? $nota['CDMUNICIBGE'] : $nota['CDESTADOIBGE'].$nota['CDMUNICIBGE']); // iif(Length(cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString)=7, cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString, cdsVendaNFCe.FieldByName('CDESTADOIBGE').AsString+cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString);
          $enderEmit->addChild('xMun', $nota['NMMUNICIPIO']); // cdsVendaNFCe.FieldByName('NMMUNICIPIO').AsString;
          $enderEmit->addChild('UF', $nota['SGESTADO']); // cdsVendaNFCe.FieldByName('SGESTADO'   ).AsString;
          $enderEmit->addChild('CEP', $nota['NRCEPIEST']); // SoNumerosStr(cdsVendaNFCe.FieldByName('NRCEPIEST').AsString);
          $enderEmit->addChild('cPais', '1058'); // '1058';
          $enderEmit->addChild('xPais', 'BRASIL'); // 'BRASIL';
        
        $emit->addChild('IE', $nota['CDINSCESTA']); // SoNumerosStr(cdsVendaNFCe.FieldByName('CDINSCESTA').AsString);
        $emit->addChild('IM', empty($nota['CDINSCMUNI']) ? 'ISENTO' : $nota['CDINSCMUNI']); // iif(EmptyString(SoNumerosStr(cdsVendaNFCe.FieldByName('CDINSCMUNI').AsString)), 'ISENTO', SoNumerosStr(cdsVendaNFCe.FieldByName('CDINSCMUNI').AsString));                 
        $emit->addChild('CNAE', $nota['CDCNAE']); // SoNumerosStr(cdsVendaNFCe.FieldByName('CDCNAE'    ).AsString);
        $emit->addChild('CRT', $nota['CDSITUCRT']); // cdsVendaNFCe.FieldByName('CDSITUCRT' ).AsString;
    }
    
    private function xmlconfigINFNFDEST($xml, $nota) {
        $parametros = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'NRSEQVENDA'], $nota);
        $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQCONSUMIDORNFCE, $parametros);
        
        if ($resposta->error()) {
            throw new \Exception("Erro ao 
            buscar consumidos. Função: xmlconfigINFNFDEST. Erro: ".$resposta->getMessage(), 4);
        } else {
            $consumidor = $resposta->getResult()[0];
            
            $stCpfCnpj = '';
            
            if (!empty($consumidor['NRINSCRCONS'])) {
                $dest = $xml->addChild('dest'); //with AddChild('dest') do begin
                if (strlen($consumidor['NRINSCRCONS']) == 11) {
                     $dest->addChild('CPF', $consumidor['NRINSCRCONS']); // cdsConsumidorNFCe.FieldByName('NRINSCRCONS').AsString;
                } else {
                     $dest->addChild('CNPJ', $consumidor['NRINSCRCONS']); // cdsConsumidorNFCe.FieldByName('NRINSCRCONS').AsString;
                }
                
                if ($nota['IDTPAMBNFCE'] == '1') {
                    if (!empty($consumidor['NMCONSVEND'])) {
                        $dest->addChild('xNome', $this->removerAcentos($consumidor['NMCONSVEND'])); // Trim(RemoveAcentos(cdsConsumidorNFCe.FieldByName('NMCONSVEND').AsString));                        
                    } else if (!empty($consumidor['NMCONSUMIDOR'])) {
                        $dest->addChild('xNome', $this->removerAcentos($consumidor['NMCONSUMIDOR'])); // Trim(RemoveAcentos(cdsConsumidorNFCe.FieldByName('NMCONSUMIDOR').AsString));                        
                    } else {
                        $dest->addChild('xNome', 'CONSUMIDOR FINAL'); // 'CONSUMIDOR FINAL';
                    }
                } else if ($nota['IDTPAMBNFCE'] == '2') {
                     $dest->addChild('xNome', 'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL'); // 'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
                }
                
                if (!empty($nota['DSENDECONSVENDA']) and strlen($nota['DSENDECONSVENDA']) > 3) {
                    $enderDest = $xml->addChild('enderDest');
                    $enderDest->addChild('xLgr', $this->removerAcentos($nota['DSENDECONSVENDA'])); // Trim(RemoveAcentos(cdsVendaNFCe.FieldByName('DSENDECONSVENDA').AsString));
                    $enderDest->addChild('nro', str_pad('', 3, "0", STR_PAD_LEFT)); // '000';
                    $enderDest->addChild('xBairro', $nota['NMBAIRIEST']); // cdsVendaNFCe.FieldByName('NMBAIRIEST' ).AsString;
                    $enderDest->addChild('cMun', strlen($nota['CDMUNICIBGE']) ); // iif(Length(cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString)=7, cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString, cdsVendaNFCe.FieldByName('CDESTADOIBGE').AsString+cdsVendaNFCe.FieldByName('CDMUNICIBGE').AsString);
                    $enderDest->addChild('xMun', strlen($nota['CDMUNICIBGE']) == 7 ? $nota['CDMUNICIBGE'] : $nota['CDESTADOIBGE'].$nota['CDMUNICIBGE']); // cdsVendaNFCe.FieldByName('NMMUNICIPIO').AsString;
                    $enderDest->addChild('UF', $nota['SGESTADO']     ); // cdsVendaNFCe.FieldByName('SGESTADO'   ).AsString;
                    $enderDest->addChild('CEP', '00000000'); // '00000000';
                    $enderDest->addChild('cPais', '1058'); // '1058';
                    $enderDest->addChild('xPais', 'BRASIL'); // 'BRASIL';
                  } //enderDest
                  $dest->addChild('indIEDest', '9');
            } //dest
        }
    }
    
    public function xmlconfigDET($xml, $notas) {
        $this->inNumProduto    = 1;
        $i = 1;
        $this->doVrTotalTxServ = 0;
        $this->doVrTotalBc     = 0;
        $this->doVrTotalBcST   = 0;
        $this->doVrTotalIcms   = 0;
        $this->doVrTotalProd   = 0;
        $this->doVrTotalDesc   = 0;
        $this->doVrTotalPis    = 0;
        $this->doVrTotalCofins = 0;
        $this->doVrTotalTrib   = 0;
        $this->doVrPisProd     = 0;
        $this->doVrTotalNFCe   = 0;
        $result = array(
            'error'     => false
        );
        
        foreach ($notas as $nota) {
            
            $this->doVrIcmsProd   = 0; // 0;
            $this->doVrBaseCalc   = 0; // 0;
            $this->doVrBaseCalcST = 0; // 0;
            $this->doPorcIcms     = $nota['IDTPIMPOSFIS'] == 'T' ? $nota['VRPEALIMPFIS'] : 0.0; // iif(cdsVendaNFCe.FieldByName('IDTPIMPOSFIS').AsString = 'T', cdsVendaNFCe.FieldByName('VRPEALIMPFIS').AsFloat, 0.00);
            $this->doVrDescVenda  = number_format($nota['VRDESITVEND'], 2, '.', ''); // Trunc2(cdsVendaNFCe.FieldByName('VRDESITVEND').AsFloat);
            $this->doVrTrib       = number_format($nota['VRTOTTRIBIBPT']/sizeof($notas), 2, '.', ''); // Trunc2( cdsVendaNFCe.FieldByName('VRTOTTRIBIBPT').AsFloat/cdsVendaNFCe.RecordCount );
               
            $det = $xml->addChild('det'); // with AddChild('det') do begin
            $det->addAttribute('nItem', (string) $i); // Attributes['nItem'] := FloatToStr(inNumProduto);
            $this->xmlconfigDETPROD($det, $nota);
            $resultDETIMPOSTO = $this->xmlconfigDETIMPOSTO($det, $nota);
            
            if ($resultDETIMPOSTO['error']) {
                $result['error'] = true;
            }
            
            if (($nota['CDCSTICMS'] == '00') or ($nota['CDCSTICMS'] == '10') or
                  ($nota['CDCSTICMS'] == '20') or ($nota['CDCSTICMS'] == '51') or
                  ($nota['CDCSTICMS'] == '70') or ($nota['CDCSTICMS'] == '90') or ($nota['CDCSTICMS'] == '900')) {
                $this->doVrTotalBc  = $this->doVrTotalBc + $this->doVrBaseCalc;
            }
            
            if ($nota['CDCSTICMS'] == '202') {
                $this->doVrTotalBcST = $this->doVrTotalBcST  + $this->doVrBaseCalcST;
            } 
            $this->doVrTotalTxServ = $this->doVrTotalTxServ + $nota['VRACRITVEND'];
            $this->doVrTotalIcms   = $this->doVrTotalIcms   + $this->doVrIcmsProd;
            $this->doVrTotalProd   = $this->doVrTotalProd   + number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'], 2, '.', '');
            $this->doVrTotalNFCe   = $this->doVrTotalNFCe   + $this->doVrTotalProd;
            $this->doVrTotalDesc   = $this->doVrTotalDesc   + $this->doVrDescVenda;
            $this->doVrTotalPis    = $this->doVrTotalPis    + $this->doVrPisProd;
            $this->doVrTotalCofins = $this->doVrTotalCofins + $this->doVrCofinsProd;
            $this->doVrTotalTrib   = $this->doVrTotalTrib   + $this->doVrTrib;
            // $this->inNumProduto++;
            $i++;
        }
        
        return $result;
        
    } 
    
    public function xmlconfigDETPROD($xml, $nota) {
        $prod = $xml->addChild('prod'); // with AddChild('prod') do begin
        
        $prod->addChild('cProd', $nota['CDARVPROD']); // cdsVendaNFCe.FieldByName('CDARVPROD' ).AsString;
        if ($nota['CDVERSAOXMLNFCE'] == '3.10') {
            $prod->addChild('cEAN', ''); // '';
        } else {
            $prod->addChild('cEAN', 'SEM GTIN'); // 'SEM GTIN';
        }
         
        if (($this->inNumProduto = 1) and ($nota['IDTPAMBNFCE'] == '2')) {
            $prod->addChild('xProd', 'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL'); // 'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
        } else {
            $valor = str_replace('+', '', $nota['NMPRODUTO']); // StringReplace(cdsVendaNFCe.FieldByName('NMPRODUTO').AsString, '+', '', [rfReplaceAll])
            $prod->addChild('xProd', $this->removerAcentos($valor)); // Trim(RemoveAcentos(StringReplace(cdsVendaNFCe.FieldByName('NMPRODUTO').AsString, '+', '', [rfReplaceAll])));
        }
        $prod->addChild('NCM', $nota['CDCLASFISC']);// cdsVendaNFCe.FieldByName('CDCLASFISC').AsString;
        if ( (!empty($nota['CDCEST'])) and
              (($nota['CDCSTICMS'] == '10' ) or ($nota['CDCSTICMS'] == '30' ) or
              ($nota['CDCSTICMS'] == '60' ) or ($nota['CDCSTICMS'] == '70' ) or
              ($nota['CDCSTICMS'] == '90' ) or ($nota['CDCSTICMS'] == '201') or
              ($nota['CDCSTICMS'] == '202') or ($nota['CDCSTICMS'] == '203') or
              ($nota['CDCSTICMS'] == '500') or ($nota['CDCSTICMS'] == '900'))) {
             
            $valor = str_replace('.', '', $nota['CDCEST']);
            $prod->addChild('CEST', $valor); // StringReplace(cdsVendaNFCe.FieldByName('CDCEST').AsString, '.', '', [rfReplaceAll]);
        }
        $prod->addChild('CFOP', $nota['CDCFOPPFIS']); // cdsVendaNFCe.FieldByName('CDCFOPPFIS').AsString;
        $prod->addChild('uCom', $nota['SGUNIDADE']); // cdsVendaNFCe.FieldByName('SGUNIDADE' ).AsString;
        $prod->addChild('qCom', number_format($nota['QTPRODVEND'], 4, '.', '')); // TrocaVirgPPto(FormatFloat('###0.0000',cdsVendaNFCe.FieldByName('QTPRODVEND').AsFloat));
        $prod->addChild('vUnCom', number_format($nota['VRUNITVEND'], 10, '.', '')); // TrocaVirgPPto(FormatFloat('###0.0000000000',cdsVendaNFCe.FieldByName('VRUNITVEND').AsFloat));
        $prod->addChild('vProd', number_format($nota['QTPRODVEND'] * $nota['VRUNITVEND'], 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', Trunc2( cdsVendaNFCe.FieldByName('QTPRODVEND').AsFloat*cdsVendaNFCe.FieldByName('VRUNITVEND').AsFloat )));
        if ($nota['CDVERSAOXMLNFCE'] == '3.10') {
            $prod->addChild('cEANTrib', ''); // '';
        } else {
            $prod->addChild('cEANTrib', 'SEM GTIN'); // 'SEM GTIN';
        }
        $prod->addChild('uTrib', $nota['SGUNIDADE']); // cdsVendaNFCe.FieldByName('SGUNIDADE' ).AsString;
        $prod->addChild('qTrib', number_format($nota['QTPRODVEND'], 4, '.', '')); // TrocaVirgPPto(FormatFloat('###0.0000',cdsVendaNFCe.FieldByName('QTPRODVEND').AsFloat));
        $prod->addChild('vUnTrib', number_format($nota['VRUNITVEND'], 10, '.', '')); // TrocaVirgPPto(FormatFloat('###0.0000000000',cdsVendaNFCe.FieldByName('VRUNITVEND').AsFloat));
        if ($this->doVrDescVenda > 0) 
            $prod->addChild('vDesc', number_format($this->doVrDescVenda, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrDescVenda));
        if ($nota['VRACRITVEND'] > 0)
            $prod->addChild('vOutro', number_format($nota['VRACRITVEND'], 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',cdsVendaNFCe.FieldByName('VRACRITVEND').AsFloat));
        $prod->addChild('indTot', '1'); // '1'; //1–O valor do item(vProd) compõe o valor total da NFCe
    }
    
    public function xmlconfigDETIMPOSTO($xml, $nota) {
        
        $produtosComErro = 0;
        
        $imposto = $xml->addChild('imposto'); // with AddChild('imposto') do begin
        
        $imposto->addChild('vTotTrib', $this->doVrTrib); //AddChild('vTotTrib').NodeValue := TrocaVirgPPto(FormatFloat('###0.00', doVrTrib));
        
        try {
            $ICMS = $imposto->addChild('ICMS'); // with AddChild('ICMS') do begin
            
            switch ($nota['CDCSTICMS']) {
                case "00":
                    $ICMS00 = $ICMS->addChild('ICMS00'); // with AddChild('ICMS00') do begin
                    $ICMS00->addChild('orig', '0' ); // '0';
                    $ICMS00->addChild('CST', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                    $ICMS00->addChild('modBC', '3'); // '3';
                    $this->doVrBaseCalc = number_format(($nota['VRUNITVEND'] * $nota['QTPRODVEND']) + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', ''); 
                    // doVrBaseCalc := Trunc2((cdsVendaNFCe.FieldByName('VRUNITVEND').AsFloat * cdsVendaNFCe.FieldByName('QTPRODVEND').AsFloat)+cdsVendaNFCe.FieldByName('VRACRITVEND').AsFloat-doVrDescVenda );
                    $ICMS00->addChild('vBC', $this->doVrBaseCalc); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalc));
                    $ICMS00->addChild('pICMS', number_format($this->doPorcIcms, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doPorcIcms));
                    $this->doVrIcmsProd = $this->roundABNT($this->doVrBaseCalc * ($this->doPorcIcms/100), 2); // RoundABNT(doVrBaseCalc * (doPorcIcms/100),2);
                    $ICMS00->addChild('vICMS', number_format($this->doVrIcmsProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doVrIcmsProd));
                break;
                case "20": 
                    $ICMS20 = $ICMS->addChild('ICMS20'); //with AddChild('ICMS20') do begin
                        $ICMS20->addChild('orig', '0'); // '0';
                        $ICMS20->addChild('CST', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                        $ICMS20->addChild('modBC', '3'); // '3';
                        $ICMS20->addChild('pRedBC', $nota['VRPERCREDUCAO']); // TrocaVirgPPto(FormatFloat('###0.00', cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat));
                        $this->doVrBaseCalc = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', ''); // Trunc2((cdsVendaNFCe.FieldByName('VRUNITVEND').AsFloat * cdsVendaNFCe.FieldByName('QTPRODVEND').AsFloat)+cdsVendaNFCe.FieldByName('VRACRITVEND').AsFloat-doVrDescVenda );
                        $this->doVrBaseCalc = number_format($this->doVrBaseCalc - ($this->doVrBaseCalc * $nota['VRPERCREDUCAO'] / 100), 2, '.', '');
                        // doVrBaseCalc := Trunc2(doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100));
                        $ICMS20->addChild('vBC', $this->doVrBaseCalc); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalc));
                        $ICMS20->addChild('pICMS', number_format($this->doPorcIcms, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doPorcIcms));
                        $this->doVrIcmsProd = $this->roundABNT($this->doVrBaseCalc * ($this->doPorcIcms/100), 2);
                        $ICMS20->addChild('vICMS', number_format($this->doVrIcmsProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doVrIcmsProd));
                break; //ICMS20
                case "40" || "41":
                    $ICMS40 = $ICMS->addChild('ICMS40'); // with AddChild('ICMS40') do begin
                    $ICMS40->addChild('orig', '0'); // '0';
                    $ICMS40->addChild('CST', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                break;
                case "60": 
                    $ICMS60 = $ICMS->addChild('ICMS60');
                        $ICMS60->addChild('orig', '0'); // '0';
                        $ICMS60->addChild('CST', $nota['CDCSTICMS'] ); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                break;
                case "90":
                    $ICMS90 = $ICMS->addChild('ICMS90'); // with AddChild('ICMS90') do begin
                    $ICMS90->addChild('orig', '0'); // '0';
                    $ICMS90->addChild('CST', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                    $ICMS90->addChild('modBC', '3'); // '3';
                    $this->doVrBaseCalc = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', ''); // Trunc2((cdsVendaNFCe.FieldByName('VRUNITVEND').AsFloat * cdsVendaNFCe.FieldByName('QTPRODVEND').AsFloat)+cdsVendaNFCe.FieldByName('VRACRITVEND').AsFloat-doVrDescVenda );
                    $this->doVrBaseCalc = number_format($this->doVrBaseCalc - ($this->doVrBaseCalc * $nota['VRPERCREDUCAO'] / 100), 2, '.', ''); 
                    // Trunc2( doVrBaseCalc - (doVrBaseCalc * cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat / 100) );
                    $ICMS90->addChild('vBC', $this->doVrBaseCalc); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalc));
                    $ICMS90->addChild('pRedBC', $nota['VRPERCREDUCAO']); // TrocaVirgPPto(FormatFloat('###0.00', cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat));
                    $ICMS90->addChild('pICMS', number_format($this->doPorcIcms, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doPorcIcms));
                    $this->doVrIcmsProd = $this->roundABNT($this->doVrBaseCalc * ($this->doPorcIcms/100),2);
                    $ICMS90->addChild('vICMS', number_format($this->doVrIcmsProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doVrIcmsProd));
                break; //ICMS90
                case "102" || "103" || "300" || "400":
                    $ICMSSN102 = $ICMS->addChild('ICMSSN102'); // with AddChild('ICMSSN102') do begin
                    $ICMSSN102->addChild('orig', '0'); // '0';
                    $ICMSSN102->addChild('CSOSN', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                break; //ICMSSN102
                case "500":
                    $ICMSSN500 = $ICMS->addChild('ICMSSN500'); //with AddChild('ICMSSN500') do begin
                    $ICMSSN500->addChild('orig', '0'); // '0';
                    $ICMSSN500->addChild('CSOSN', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                break; //ICMSSN500
                case "900":
                    $ICMSSN900 = $ICMS->addChild('ICMSSN900'); //with AddChild('ICMSSN900') do begin
                    $ICMSSN900->addChild('orig', '0'); // '0';
                    $ICMSSN900->addChild('CSOSN', $nota['CDCSTICMS']); // cdsVendaNFCe.FieldByName('CDCSTICMS').AsString;
                    $ICMSSN900->addChild('modBC', '3'); // '3';
                    $this->doVrBaseCalc = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', '');
                    $this->doVrBaseCalc = number_format($this->doVrBaseCalc - ($this->doVrBaseCalc * $nota['VRPERCREDUCAO'] / 100), 2, '.', '');
                    $ICMSSN900->addChild('vBC', $this->doVrBaseCalc); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalc));
                    $ICMSSN900->addChild('pRedBC', $nota['VRPERCREDUCAO']); // TrocaVirgPPto(FormatFloat('###0.00', cdsVendaNFCe.FieldByName('VRPERCREDUCAO').AsFloat));
                    $ICMSSN900->addChild('pICMS', number_format($this->doPorcIcms, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doPorcIcms));
                    $this->doVrIcmsProd = $this->roundABNT($doVrBaseCalc * ($this->doPorcIcms/100),2);
                    $ICMSSN900->addChild('vICMS', number_format($this->doVrIcmsProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doVrIcmsProd));
                 end;
                break; //ICMSSN900
            }
        } catch (\Exception $e) {
            $produtosComErro++;
            TransmissaoNfceController::escreverLog("Produto: ". $nota['CDPRODUTO'] ." - ". $nota['NMPRODUTO'] ." sem ICMS cadastrado corretamente! (Verificar CST e Alíquota).");
            TransmissaoNfceController::escreverLog($e->getMessage());
            // throw new \Exception("Produto: ". $nota['CDPRODUTO'] ." - ". $nota['NMPRODUTO'] ." sem ICMS cadastrado corretamente! (Verificar CST e Alíquota).", 1);
        }
        
        $this->doVrPisProd    = 0;
        $this->doVrCofinsProd = 0;

       /* Código que existe no delphi, mas não entendi.
       try {
            $nota['VRALIQPIS'] = str_replace($nota['VRALIQPIS'], '.', ',');
            $nota['VRALIQCOFINS'] = str_replace($nota['VRALIQCOFINS'], '.', ',');
        } catch (\Exception $e) {
            $nota['VRALIQPIS'] = str_replace($nota['VRALIQPIS'], ',', '.');
            $nota['VRALIQCOFINS'] = str_replace($nota['VRALIQCOFINS'], ',', '.');
        }*/

        try {
            $PIS = $imposto->addChild('PIS'); // with AddChild('PIS') do begin
            
            switch ($nota['CDCSTPISCOF']) { //case StrToInt(cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString) of
                case "01" || "02":
                    $PISAliq = $PIS->addChild('PISAliq'); // with AddChild('PISAliq') do begin
                    $PISAliq->addChild('CST', $nota['CDCSTPISCOF']); // cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString;
                    $doVrBaseCalcPis = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', '');
                    $PISAliq->addChild('vBC', $doVrBaseCalcPis); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalcPis));
                    if (gettype($nota['VRALIQPIS']) == 'string') {
                        $nota['VRALIQPIS'] = $this->utilService->moneyToDouble($nota['VRALIQPIS']); 
                    }
                    $PISAliq->addChild('pPIS', number_format($nota['VRALIQPIS'], 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', StrToFloat(cdsVendaNFCe.FieldByName('VRALIQPIS').AsString)));
                    $this->doVrPisProd = $this->roundABNT($doVrBaseCalcPis * ($nota['VRALIQPIS']/100), 2); // RoundABNT(doVrBaseCalcPis * (StrToFloat(cdsVendaNFCe.FieldByName('VRALIQPIS').AsString)/100),2);
                    $PISAliq->addChild('vPIS', $this->doVrPisProd); // TrocaVirgPPto(FormatFloat('###0.00', $this->doVrPisProd));
                    break; //PISAliq
                case "04" || "05" || "06" || "07" || "08" || "09":
                    $PISNT = $PIS->addChild('PISNT'); // with AddChild('PISNT') do begin
                    $PISNT->addChild('CST', $nota['CDCSTPISCOF']); // cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString;
                    break;  //PISNT
                case "99":
                    $PISOutr = $PIS->addChild('PISOutr'); //with AddChild('PISOutr') do begin
                    $PISOutr->addChild('CST', $nota['CDCSTPISCOF']); // cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString;
                    $doVrBaseCalcPis = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', '');
                    $PISOutr->addChild('vBC', $doVrBaseCalcPis); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalcPis));
                    $PISOutr->addChild('pPIS', $nota['VRALIQPIS']); // TrocaVirgPPto(FormatFloat('###0.00', StrToFloat(cdsVendaNFCe.FieldByName('VRALIQPIS').AsString)));
                    $this->doVrPisProd =$this->roundABNT($doVrBaseCalcPis * ($nota['VRALIQPIS']/100), 2);
                    $PISOutr->addChild('vPIS', $this->doVrPisProd); // TrocaVirgPPto(FormatFloat('###0.00', doVrPisProd));
                    break; //PISOutr
            }

            $COFINS = $imposto->addChild('COFINS'); //with AddChild('COFINS') do begin
            
            switch ($nota['CDCSTPISCOF']) {
                case "01" || "02": 
                    $COFINSAliq = $COFINS->addChild('COFINSAliq');
                    $COFINSAliq->addChild('CST', $nota['CDCSTPISCOF']); // cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString;
                    $doVrBaseCalcCofins = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', '');
                    $COFINSAliq->addChild('vBC', $doVrBaseCalcCofins); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalcCofins));
                    if (gettype($nota['VRALIQCOFINS']) == 'string') {
                        $nota['VRALIQCOFINS'] = $this->utilService->moneyToDouble($nota['VRALIQCOFINS']); 
                    }
                    $COFINSAliq->addChild('pCOFINS', number_format($nota['VRALIQCOFINS'], 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', StrToFloat(cdsVendaNFCe.FieldByName('VRALIQCOFINS').AsString)));
                    $this->doVrCofinsProd = $this->roundABNT($doVrBaseCalcCofins * ((float)($nota['VRALIQCOFINS'])/100),2);
                    $COFINSAliq->addChild('vCOFINS', number_format($this->doVrCofinsProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doVrCofinsProd));
                    break; //COFINSAliq

                case "04" || "05" || "06" || "07" || "08" || "09":
                    $COFINSNT = $COFINS->addChild('COFISNT'); // with AddChild('COFINSNT') do begin
                    $COFINSNT->addChild('CST', $nota['CDCSTPISCOF']); // cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString;
                break; //COFINSNT
                case "99": 
                    $COFINSOutr = $COFINS->addChild('COFINSOutr'); //with AddChild('COFINSOutr') do begin
                    $COFINSOutr->addChild('CST', $nota['CDCSTPISCOF']); // cdsVendaNFCe.FieldByName('CDCSTPISCOF').AsString;
                    $doVrBaseCalcCofins = number_format($nota['VRUNITVEND'] * $nota['QTPRODVEND'] + $nota['VRACRITVEND'] - $this->doVrDescVenda, 2, '.', '');
                    $COFINSOutr->addChild('vBC', $doVrBaseCalcCofins); // TrocaVirgPPto(FormatFloat('###0.00', doVrBaseCalcCofins));
                    $COFINSOutr->addChild('pCOFINS', $nota['VRALIQCOFINS']); // TrocaVirgPPto(FormatFloat('###0.00', StrToFloat(cdsVendaNFCe.FieldByName('VRALIQCOFINS').AsString)));
                    $this->doVrCofinsProd = $this->roundABNT($doVrBaseCalcCofins * ((float)($nota['VRALIQCOFINS'])/100),2);
                    $COFINSOutr->addChild('vCOFINS', number_format($this->doVrCofinsProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00', doVrCofinsProd));
                    
                    break; //COFINSOutr
            }
                  
        }  catch (\Exception $e) {
            $produtosComErro++;
            TransmissaoNfceController::escreverLog("Produto: ". $nota['CDPRODUTO'] ." - ". $nota['NMPRODUTO'] ." sem PIS/COFINS cadastrado corretamente! (Verificar CST e Alíquota).");
            TransmissaoNfceController::escreverLog($e->getMessage());
            // throw new \Exception('Produto: '.$nota['CDPRODUTO'].' - '.$nota['NMPRODUTO'].' sem PIS/COFINS cadastrado corretamente! (Verificar CST e Alíquota).');
        }
        
        $result = array(
            'error'   => false
        );
        
        if ($produtosComErro > 0) {
            $result['error'] = true;
        }
        
        return $result;
    }
    
    public function removerAcentos($string) {
        $string = preg_replace("/[áàâãä]/", "a", $string);
        $string = preg_replace("/[ÁÀÂÃÄ]/", "A", $string);
        $string = preg_replace("/[éèê]/", "e", $string);
        $string = preg_replace("/[ÉÈÊ]/", "E", $string);
        $string = preg_replace("/[íì]/", "i", $string);
        $string = preg_replace("/[ÍÌ]/", "I", $string);
        $string = preg_replace("/[óòôõö]/", "o", $string);
        $string = preg_replace("/[ÓÒÔÕÖ]/", "O", $string);
        $string = preg_replace("/[úùü]/", "u", $string);
        $string = preg_replace("/[ÚÙÜ]/", "U", $string);
        $string = preg_replace("/ç/", "c", $string);
        $string = preg_replace("/Ç/", "C", $string);
        $string = preg_replace("/[][><}{)(:;,!?*%~^`@]/", "", $string);
        // $string = preg_replace("/ /", "_", $string);
        return trim($string);
    }
    
    public function roundABNT($valor, $casasdecimais) {
        
        try {
        /*Posicao do Separador no Numero*/
		$a = strstr($valor, '.');
		$separador = substr($a, 0, 1);
		$decimais = substr($a, 1);
		
		/*Verifica o tamanho das casas decimais, para saber em quantas posicoes o numero deve ser ajustado*/	
		$ajusta = strlen($decimais) > $casasdecimais + 2 ? strlen($decimais) : $casasdecimais + 2;
		
		/*Casas Decimais*/
		$decimais = str_pad($decimais, $ajusta, 0);
		
		/*Segundo Digito apos o separador*/
		$ultimos = substr($decimais, $casasdecimais+1);
		$ultimos = !empty($ultimos) ? $ultimos : 0;
		
		/*Primeiro Digito apos o separador*/
		$sub = substr($decimais, $casasdecimais, 1);
		
		/*Ultimo Digito a ser Conservado, pela quantidade de casas decimais*/
		$digitoconservado = substr($decimais, $casasdecimais-1, 1);
		
		if ($sub < 5) {
			$retorna = self::truncate($valor, $casasdecimais);
		} elseif ($sub > 5 or ($sub == 5 and $ultimos > 0)) {
			$retorna = self::truncate(substr($valor, 0, $separador + $decimais - strlen($ultimos)).'.'.(substr($decimais, 0, $casasdecimais)) + ('0'.'.'.str_pad('1', $casasdecimais, 0, STR_PAD_LEFT)), $casasdecimais);
		} elseif ($sub == 5 and $ultimos == 0 and $digitoconservado % 2 == 0) {
			$retorna = self::truncate($valor, $casasdecimais);
		} elseif ($sub == 5 and $ultimos == 0) {
			$retorna = self::truncate(substr($valor, 0, $separador + $decimais - strlen($ultimos)).'.'.(substr($decimais, 0, $casasdecimais)) + ('0'.'.'.str_pad('1', $casasdecimais, 0, STR_PAD_LEFT)), $casasdecimais);
		}
        } catch(\Exception $e) {
            $retorna = number_format($Valor, $Decimais, '.', '');
        }
		
		return $retorna;
       
    }
    
    public function truncate($val, $f="0") {
		if(($p = strpos($val, '.')) !== false) {
			$val = floatval(substr($val, 0, $p + 1 + $f));
		}
		return $val;
	}
    
    public function modulo11($numero) {
        $result = (string) $numero;
        
        try {
          $soma = 0; $k = 2;
          for ($i = strlen($numero); $i > 0; $i--) {
             $soma = $soma + (intval($numero[$i-1]) * $k);
             $k++;
             if ($k > 9) {
                $k = 2;
             }
          }
          $digito = 11 - ($soma % 11);
          if ($digito >= 10) {
             $digito = 0;
          }
          $result = $result.chr($digito + ord('0'));
        } catch (\Exception $e) {
            $result = $result.'X';
        }
        return $result;
    }
    
    //aqui
    public function xmlconfigTOTAL($xml, $nota) {
        
        $total = $xml->addChild('total'); // with AddChild('total') do begin
        
        $ICMSTot = $total->addChild('ICMSTot'); // with AddChild('ICMSTot') do begin
        $ICMSTot->addChild('vBC', number_format($this->doVrTotalBc, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalBc));
        $ICMSTot->addChild('vICMS', number_format($this->doVrTotalIcms, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalIcms));
        $ICMSTot->addChild('vICMSDeson', '0.00'); // '0.00';
        
        if ($nota['CDVERSAOXMLNFCE'] != '3.10')
            $ICMSTot->addChild('vFCP', '0.00'); // '0.00';
        $ICMSTot->addChild('vBCST', number_format($this->doVrTotalBcST, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalBcST));
        $ICMSTot->addChild('vST', '0.00'); // '0.00';
        if ($nota['CDVERSAOXMLNFCE'] != '3.10') {
            $ICMSTot->addChild('vFCPST', '0.00'); // '0.00';
            $ICMSTot->addChild('vFCPSTRet', '0.00'); // '0.00';
        }
        $ICMSTot->addChild('vProd', number_format($this->doVrTotalProd, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalProd));
        $ICMSTot->addChild('vFrete', '0.00'); // '0.00';
        $ICMSTot->addChild('vSeg', '0.00'); // '0.00';
        $ICMSTot->addChild('vDesc', number_format($this->doVrTotalDesc, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalDesc));
        $ICMSTot->addChild('vII', '0.00'); // '0.00';
        $ICMSTot->addChild('vIPI', '0.00'); // '0.00';
        if ($nota['CDVERSAOXMLNFCE'] != '3.10')
            $ICMSTot->addChild('vIPIDevol', '0.00'); // '0.00';                  
        $ICMSTot->addChild('vPIS', number_format($this->doVrTotalPis, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalPis));
        $ICMSTot->addChild('vCOFINS', number_format($this->doVrTotalCofins, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalCofins));
        $ICMSTot->addChild('vOutro', number_format($this->doVrTotalTxServ, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalTxServ));
        $ICMSTot->addChild('vNF', number_format($this->doVrTotalNFCe + $this->doVrTotalTxServ, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalNFCe + doVrTotalTxServ));
        $ICMSTot->addChild('vTotTrib', number_format($this->doVrTotalTrib, 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',doVrTotalTrib));

        $transp = $xml->addChild('transp'); //with AddChild('transp') do begin
        $transp->addChild('modFrete', '9'); //9-Sem Ocorrência de transporte
    }
    
    public function xmlconfigPAG($xml, $nota) {
        
        $cdsMovCaixaNFCe = array();
        $parametros = $this->gerarParametros(['NRSEQVENDA', 'CDFILIAL', 'CDCAIXA'], $nota);
        
        if ($nota['IDSITUVENDA'] == 'C') {
            $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQRECCANCNFCE, $parametros);
        } else {
            if ($nota['CDVERSAOXMLNFCE'] == '3.10') {
                $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQRECEBNFCE310, $parametros);
            } else {
                $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQRECEBIMENTONFCE, $parametros);
            }
        }
        
        if ($resposta->error()) {
            throw new \Exception("Erro ao buscar dados. Função: xmlconfigPAG", 10);
        } else {
            $cdsMovCaixaNFCe = $resposta->getResult();
        }

        if ($nota['CDVERSAOXMLNFCE'] == '3.10') {
           
           foreach ($cdsMovCaixaNFCe as $movcaixa) {// while not cdsMovCaixaNFCe.Eof do begin
                $pag = $xml->addChild('pag');  // with AddChild('pag') do begin                  
                if ($movcaixa['IDTIPORECE'] == '4')
                    $pag->addChild('tPag', '01'); // '01'; //Dinheiro
                else if ($movcaixa['IDTIPORECE'] == '3')
                    $pag->addChild('tPag', '02'); // '02'; //Cheque
                else if ($movcaixa['IDTIPORECE'] == '1')
                    $pag->addChild('tPag', '03'); // '03'; //Cartao de Credito
                else if ($movcaixa['IDTIPORECE'] == '2')
                    $pag->addChild('tPag', '04'); // '04'; //Cartao de Debito
                else if ($movcaixa['IDTIPORECE'] == '9')
                    $pag->addChild('tPag', '05'); // '05'; //Crédito Loja
                else
                    $pag->addChild('tPag', '99'); // '99'; //Outros
                
                $pag->addChild('vPag', number_format($movcaixa['VRMOVIVEND'], 2, '.', '')); // TrocaVirgPPto(FormatFloat('###0.00',cdsMovCaixaNFCe.FieldByName('VRMOVIVEND').AsFloat));

                if ($movcaixa['IDTIPORECE'] == '1' or $movcaixa['IDTIPORECE'] == '2') {
                    $card = $xml->addChild('card'); // with AddChild('card') do begin
                    
                    if ($movcaixa['IDDESABTEF'] = 'S') 
                        $card->addChild('tpIntegra', '2');  // '2'; //2=POS
                    else {
                        $card->addChild('tpIntegra', '1');  // '1'; //1=TEF

                        if (!empty($movcaixa['CDADMINCART']))
                            $card->addChild('CNPJ', $movcaixa['CDADMINCART']);  // SoNumerosStr($movcaixa['CDADMINCART'])
                        else 
                            $card->addChild('CNPJ', '00000000000000');  // '00000000000000';
    
                        if (!empty($movcaixa['CDBANCARTSEFAZ']))
                            $card->addChild('tBand', $movcaixa['CDBANCARTSEFAZ']);
                        else 
                            $card->addChild('tBand', '99');  // '99';
    
                        if (!empty($movcaixa['CDNSUHOSTTEF']))
                            $card->addChild('cAut', substr($movcaixa['CDNSUHOSTTEF'], 0, 19));  // Copy($movcaixa['CDNSUHOSTTEF'],1,20)
                        else 
                            $card->addChild('cAut', '000000000');  // '000000000';
                   }
                }
           } //WHILE cdsMovCaixaNFCe.Eof                  

        } else {            

            $pag = $xml->addChild('pag'); //with AddChild('pag') do begin    
            $detPag = $pag->addChild('detPag'); // with AddChild('detPag') do begin
            
              
            foreach ($cdsMovCaixaNFCe as $movcaixa) { //while not cdsMovCaixaNFCe.Eof do begin
            
                if ($movcaixa['IDTIPOMOVIVE'] != 'S') {
                
                    if ($movcaixa['IDTIPORECE'] == '4')
                        $detPag->addChild('tPag', '01'); //Dinheiro
                    else if ($movcaixa['IDTIPORECE'] == '3')
                        $detPag->addChild('tPag', '02'); //Cheque
                    else if ($movcaixa['IDTIPORECE'] == '1')
                        $detPag->addChild('tPag', '03'); //Cartao de Credito
                    else if ($movcaixa['IDTIPORECE'] == '2')
                        $detPag->addChild('tPag', '04'); //Cartao de Debito
                    else if ($movcaixa['IDTIPORECE'] == '9')
                        $detPag->addChild('tPag', '05'); //Crédito Loja
                    else 
                        $detPag->addChild('tPag', '99'); //Outros
                        
                    $detPag->addChild('vPag', number_format($movcaixa['VRMOVIVEND'], 2, '.', ''));
    
                    if ($movcaixa['IDTIPORECE'] == '1' or $movcaixa['IDTIPORECE'] == '2'){
                        $card = $detPag->addChild('card'); // with AddChild('card') do begin
                        
                        if ($movcaixa['IDDESABTEF'] = 'S') 
                            $card->addChild('tpIntegra', '2');  // '2'; //2=POS
                        else {
                            $card->addChild('tpIntegra', '1');  // '1'; //1=TEF
                            
                            if (!empty($movcaixa['CDADMINCART']))
                                $card->addChild('CNPJ', $movcaixa['CDADMINCART']);  // SoNumerosStr($movcaixa['CDADMINCART'])
                            else 
                                $card->addChild('CNPJ', '00000000000000');  // '00000000000000';
    
    
                            if (!empty($movcaixa['CDBANCARTSEFAZ']))
                                $card->addChild('tBand', $movcaixa['CDBANCARTSEFAZ']);
                            else 
                                $card->addChild('tBand', '99');  // '99';
    
                            if (!empty($movcaixa['CDNSUHOSTTEF']))
                                $card->addChild('cAut', $movcaixa['CDNSUHOSTTEF']);
                            else 
                                $card->addChild('cAut', '000000000');  // '000000000';
                        } //card;

                    }
                    
                }  //detPag
                
            } //WHILE cdsMovCaixaNFCe.Eof

            // $trocoKey = array_search('S', array_column($cdsMovCaixaNFCe, 'IDTIPOMOVIVE'));
            // if (!empty($cdsMovCaixaNFCe[$trocoKey]))
            //     $detPag->addChild('vTroco', $cdsMovCaixaNFCe[$trocoKey]['VRMOVIVEND']); // TrocaVirgPPto(FormatFloat('###0.00',cdsMovCaixaNFCe.FieldByName('VRMOVIVEND').AsFloat));
            // else
            //     $detPag->addChild('vTroco', '0.00'); // '0.00';
               
        } //END IF
    }
    
    public function xmlconfigINFADIC($xml, $nota) {
        $stTpEmis = $nota['SGESTADO'] == 'SP' ? '1' : '9';
        $infAdic = $xml->addChild('infAdic');
        if ($stTpEmis == '9') {
            $infCpl = $infAdic->addChild('infCpl', 'NFCe EMITIDA EM CONTIGENCIA');
            $obsCont = $infAdic->addChild('obsCont');
            $obsCont->addAttribute('xCampo', 'NrSeqVen-IdOrigemVen');
            $obsCont->addChild('xTexto', $nota['CDFILIAL'].'-'.$nota['CDCAIXA'].'-'.$nota['NRSEQVENDA'].'-'.$nota['IDORIGEMVENDA']);
        }
    }
    
    public function atualizarXMLBanco($xml, $nota, $qrCode = null) {
        
        $xml = str_replace('<?xml version="1.0" encoding="UTF-8"?>', '', $xml);
        $xml = str_replace('<?xml version="1.0"?>', '', $xml);
        
        $parametros = array(
            ':DSARQXMLNFCE' => $xml,
            ':NRORG'        => $nota['NRORG'],
            ':NRSEQVENDA'   => $nota['NRSEQVENDA'],
            ':CDCAIXA'      => $nota['CDCAIXA'],
            ':CDFILIAL'     => $nota['CDFILIAL'],
            ':DSQRCODENFCE' => $qrCode
        );
        $this->transmissaoService->updateVendaNFCE($parametros);
    }
    
    public function atualizaNFCE($duplicidades) {
        TransmissaoNfceController::escreverLog("Atualizando Notas Fiscais...");
        TransmissaoNfceController::escreverLog("");
        $time_start = microtime(true);
        
        foreach ($this->notasValidas as $nota) {
            
            $parametros['CDFILIAL']     = $nota['CDFILIAL'];
            $parametros['CDCAIXA']      = $nota['CDCAIXA'];
            $parametros['NRSEQVENDA']   = $nota['NRSEQVENDA'];
            
            //Verifica se nota encontra-se no array das notas validas
            $item = array_filter(
                $duplicidades,
                function ($e) use ($nota) {
                    return $e['CDFILIAL'] == $nota['CDFILIAL'] && $e['CDCAIXA'] == $nota['CDCAIXA'] && $e['NRSEQVENDA'] == $nota['NRSEQVENDA'];
                }
            );
            
            if (!empty($item) && sizeof($item) == 1) { 
                $item = array_shift($item);
                TransmissaoNfceController::escreverLog("Unidade/Filial: ".$item['CDFILIAL']. " Caixa: ".$item['CDCAIXA']. " Venda: ".$item['NRSEQVENDA']." Nota: ".$item['NRNOTAFISCALCE']);
                $getNumeroDuplicidade = $this->getNumeroDuplicidade($item['DSOBSSTATUSNFCE']);
                if(!empty($getNumeroDuplicidade['chave'])){ 
                    $chaveAcesso = $getNumeroDuplicidade['chave'];
                    $parametros['NRACESSONFCE'] = $chaveAcesso;
                    $parametros['NRRECIBONFCE'] = (!empty($getNumeroDuplicidade['recibo'])) ? $getNumeroDuplicidade['recibo'] : $item['NRRECIBONFCE'];
                    //devera consultar chave de acesso
                }else if(!empty($getNumeroDuplicidade['recibo'])){
                    $nRecibo = $getNumeroDuplicidade['recibo'];
                    TransmissaoNfceController::escreverLog("Consulta NFCe através do Recibo");
                    $result = $this->notaFiscal->consultaReciboNFe($nRecibo,$nota);
                    if($result->cStat == 104){
                        TransmissaoNfceController::escreverLog(" # Lote processado");
                        if(isset($result->chNFe)){ 
                            $parametros['NRACESSONFCE'] = $result->chNFe;
                            $parametros['NRRECIBONFCE'] = $result->nRec;
                        }else{
                            TransmissaoNfceController::escreverLog(" Não foi possível recuperar informações da NFCe");
                            TransmissaoNfceController::escreverLog("");
                        }
                    }else{
                        TransmissaoNfceController::escreverLog(" # Lote não processado. ". (isset($result->cStat)) ? $result->cStat : "" . "  ". (isset($result->xMotivo)) ? $result->xMotivo : "");
                        TransmissaoNfceController::escreverLog("");
                    } 
                }else{
                    
                    //nao encontrou as informacoes de chave de acesso e nem lote, recalcula chave e busca informacoes
                    $nota['NRACESSONFCE'] = $this->notaFiscal->recalculaChave($item['NRCHAVENOTA']);
                    $result = $this->notaFiscal->consultaSituacaoNFe($nota);
                    $xml = $result;
                    $status = $this->getParameterXML($xml, 'cStat');
                    $nProt = $this->getParameterXML($xml, 'nProt');
                    $chNFe = $this->getParameterXML($xml, 'chNFe');
                    
                    if($status == 100 || $status == 150){
                        $parametros['NRACESSONFCE'] = $chNFe;
                        $parametros['NRRECIBONFCE'] = $item['NRRECEBIDO'];
                    }
                }
                
                //apos realizar a consulta das informacoes corretas
                if(isset($parametros['NRACESSONFCE'])){
                      //define parametros 
                      $prt = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'CDLOJA', 'NRSEQVENDA'], $nota);
                      $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQTRANSMISSAONFCE, $prt);
                      $resposta = $resposta->getResult()[0];
                      if (substr($parametros['NRACESSONFCE'], 0, 2) != '52') {
                        $xml =  $this->regerarXMLValido($parametros['NRACESSONFCE'], $parametros['NRRECIBONFCE'], $nota);
                        if (!empty($xml)) {
                            $xmlAssinado = $this->notaFiscal->assinarXMLExterno($xml, $resposta);
                            //TransmissaoNfceController::escreverLog($xmlAssinado);
                            if($xmlAssinado){
                                $getQrCode = $this->getQrCode($xmlAssinado);
                                //realiza update dos campos NRACESSONFCE e NRRECIBONFCE
                                TransmissaoNfceController::escreverLog("Atualizando Chave de Acesso e Recibo da venda.");
                                $this->transmissaoService->runQuery(TransmissaoNfceService::ACMDUPLICIDADES, $parametros);
                            }
                            TransmissaoNfceController::escreverLog("Atualizando dados da venda.");
                            TransmissaoNfceController::escreverLog("");
                            $nota['NRACESSONFCE'] = $parametros['NRACESSONFCE'];
                            $this->atualizarXMLBanco($xmlAssinado, $nota, $getQrCode);
                        }
                    } else {
                        //define parametros 
                        $parametros = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'CDLOJA', 'NRSEQVENDA'], $nota);
                        $parametros['NRACESSONFCE'] = $this->modulo11(substr($nota['NRACESSONFCE'], 0, 33) + '1' + substr($nota['NRACESSONFCE'], 35, 43));
                        $this->transmissaoService->runQuery(TransmissaoNfceService::ACMDUPLICIDADESGOIAS, $parametros);
                    }
                    //Verifica situacao da NFCe diante das atualizacoes de chave de acesso
                    $this->situacaoNFCe($nota);
                }else{
                    TransmissaoNfceController::escreverLog("Não foi possível encontrar as informações da Nota!");
                    TransmissaoNfceController::escreverLog("");
                }
            }
        }
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
        TransmissaoNfceController::escreverLog("");
    }
    
    private function getParameterXML($xml, $parametro) {
        
        $chave1 = '<'.$parametro.'>';
        $chave2 = '</'.$parametro.'>';
        
        if(strpos($xml, $chave1) && strpos($xml, $chave2)){
            $posIn = strpos($xml, $chave1);
            $posFi = strpos($xml, $chave2);
            $dado = substr($xml, $posIn+strlen($chave1), $posFi-($posIn+strlen($chave1)));
            return $dado;
        }else{
            return null;
        }
    }
    
    private function getNumeroDuplicidade($texto){
        $result = array(
            "chave" => '',
            "recibo" => ''
        );
        $cortado = array();
        preg_match('/\[(.*?)\]/', $texto, $cortado);
        
        if(!empty($cortado)){
            $posChave = strpos($cortado[1], 'chNFe:');
            $posRecibo = strpos($cortado[1], 'nRec:');
            if (!($posChave === false) && ($posRecibo === false)) {
                $tamanho = strlen('chNFe:');
                $result['chave'] = substr($cortado[1], $tamanho, 44);
            }else if(strlen($cortado[1]) == 44){
                $result['chave'] = $cortado[1];
            }
            if (!($posRecibo === false)) {
                $tamanho = strlen('nRec:');
                $result['recibo'] = substr($cortado[1], $tamanho, 15);
            }else if(strlen($cortado[1]) == 15){
                $result['recibo'] = $cortado[1];
            }
        }
        return $result;
    }
    
    
    private function getQrCode($signedXML){
		$begin = strpos($signedXML, '<qrCode>');
		if ($begin) {
			$beginStr = substr($signedXML, $begin + 17);
			$end = strpos($beginStr, '</qrCode>');
			$qrCode = substr($beginStr, 0, $end - 3);
		} else {
			$qrCode = "QRCode inexistente.";
		}
		return $qrCode;
	}
    
    private function regerarXMLValido($chave, $recibo = null, $nota, $dadosRetorno = null){
        
        $xml = simplexml_load_string($nota['DSARQXMLNFCE']);
        $arrayData = $this->xmlToArray($xml);
        
        //troca a chave de acesso de referencia da nota fiscal
        $arrayData['NFe']['infNFe']['@Id'] = 'NFe'.$chave;
        
        //verifica chave de acesso - se foi emissao normal ou contigencia 
        $tpEmis = substr($chave, -10, 1);
        $arrayData['NFe']['infNFe']['ide']['tpEmis'] = $tpEmis;
        
        //verifica digitoVerificador
        $cDV = $this->notaFiscal->calculaDigitoVerificadorNFe($chave); 
        $arrayData['NFe']['infNFe']['ide']['cDV'] = $cDV;
        
        //se o tpEmissao for 9, preencher informacoes, caso nao, remover
        if($tpEmis != 9){
            unset($arrayData['NFe']['infNFe']['ide']['dhCont']);
            unset($arrayData['NFe']['infNFe']['ide']['xJust']);
        }
        
        //informacoes complementares (verificar como essa tag fica no FORSALE)
        //$arrayData['NFe']['infNFe']['infAdic']['infCpl'] = '';
        //$arrayData['NFe']['infNFe']['infAdic']['obsCont']['xCampo'] = 'NrSeqVen-IdOrigemVen';
        //$arrayData['NFe']['infNFe']['infAdic']['obsCont']['xTexto'] = '0000175373-BAL_FNC';
        
        //remover qrcode, pq sera gerado novamente
        unset($arrayData['NFe']['infNFeSupl']);
        unset($arrayData['NFe']['Signature']);
        
        $xmlRetorno = $this->arrayToXml($arrayData['NFe']);
        return $xmlRetorno;      
    }
    
    private function situacaoNFCe($nota) {
        $result = $this->notaFiscal->consultaSituacaoNFe($nota);
        $xml = $result;
        
        $status = $this->getParameterXML($xml, 'cStat');
        $nProt = $this->getParameterXML($xml, 'nProt');
        $chNFe = $this->getParameterXML($xml, 'chNFe');
        
        /* 100 - Autorizado o uso da NF-e
           101 - Cancelamento de NF-e homologado
           110 - Uso Denegado
           150 - Autorizado o uso da NF-e, autorização concedida fora do prazo 
           217 - NF-e não consta na base de dados da SEFAZ
        */
        
       if (($status == '100') or ($status == '101') or ($status == '110') or ($status == '150')) {
            $dhRecbto = $this->getParameterXML($xml, 'dhRecbto');
            $dtHrProtocoNFCe = substr($dhRecbto, 0, 10).' '.substr($dhRecbto, 11, 8);
            $dtHrProtocoNFCe = date_create_from_format('Y-m-d H:s:i', $dtHrProtocoNFCe);
            $dtHrProtocoNFCe = $dtHrProtocoNFCe->format('d/m/Y H:s:i');
            
            
            if ($status == '100' or $status == '150') { 
                $stStatusNFe = 'A';
            } else {
                if ($status = '101')
                    $stStatusNFe = 'C';
                if ($status = '110')
                    $stStatusNFe = 'D';
            }
            
            $xMotivo = strlen($this->getParameterXML($xml, 'xMotivo')) > 3392 ? substr($this->getParameterXML($xml, 'xMotivo'), 0, 3393) : $this->getParameterXML($xml, 'xMotivo');
            
            //Copia-se 3933 caracteres do XML, pois a observação ocupa mais 67 caracteres
            $parametros['IDSTATUSNFCE'   ] = $stStatusNFe;
            $parametros['DSOBSSTATUSNFCE'] = $this->getParameterXML($xml, 'cStat') . ' - ' . $xMotivo . ' (Nota atualizada pelo processo de Consulta Situação de NFCe - Correção de Duplicidade)';
            $parametros['NRPROTOCOLONFCE'] = $this->getParameterXML($xml, 'nProt');
            $parametros['DTHRPROTOCONFCE'] = $dtHrProtocoNFCe;
            $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
            $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
            $parametros['NRSEQVENDA'     ] = $nota['NRSEQVENDA'];
            $parametros['NRACESSONFCE'   ] = $chNFe;
            TransmissaoNfceController::escreverLog("Atualizando situacao atual - Nota Fiscal ".$nota['NRNOTAFISCALCE']." - através da chave de acesso. Status: ".$status);
            TransmissaoNfceController::escreverLog("");
            TransmissaoNfceController::escreverLog("Atualizando tabela...");
            $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDPROTOCNFCE, $parametros);
            
            if (($nota['IDSITUVENDA'] == 'C') and ($stStatusNFe == 'A')) {
                $venda = array(
                    'NRACESSONFCE'    => $chNFe,
                    'NRPROTOCOLONFCE' => $nota['NRPROTOCOLONFCE']
                );
                TransmissaoNfceController::escreverLog("Alterando informações...");
                $this->notaFiscal->cancelaVendaNFCE($nota['CDFILIAL'], $nota['CDCAIXA'], $nota['NRORG'], $nota['NRSEQVENDA'], $venda, $nota['DSRAZAOCANCNFCE'], $nota['CDOPERADOR']);
                TransmissaoNfceController::escreverLog("Operação realizada.");
            }
            TransmissaoNfceController::escreverLog("");
            
        } else if ($status != '217') {
            //Copia-se 3933 caracteres do XML, pois a observação ocupa mais 67 caracteres
            $xMotivo = strlen($this->getParameterXML($xml, 'xMotivo')) > 3392 ? substr($this->getParameterXML($xml, 'xMotivo'), 0, 3393) : $this->getParameterXML($xml, 'xMotivo');
            $parametros['IDSTATUSNFCE'   ] = 'R';
            $parametros['DSOBSSTATUSNFCE'] = $this->getParameterXML($xml, 'cStat').' - ' . $xMotivo . ' (Nota atualizada pelo processo de Consulta Situação de NFCe - Correção de Duplicidade)';
            $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
            $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
            $parametros['NRSEQVENDA'     ] = $nota['NRSEQVENDA'];  
            TransmissaoNfceController::escreverLog("Atualizando situacao atual - Nota Fiscal ".$nota['NRNOTAFISCALCE']." - através da chave de acesso. Status: ".$status);
            TransmissaoNfceController::escreverLog("");
            TransmissaoNfceController::escreverLog("Atualizando tabela...");
            $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDNFCE, $parametros);
        }
        
        if(isset($parametros['DSOBSSTATUSNFCE'])){
           TransmissaoNfceController::escreverLog("Observação: ".$parametros['DSOBSSTATUSNFCE']);     
        }else{
           TransmissaoNfceController::escreverLog("");
        }
        return true;
    }
    
    //$test = false;
    private function atualizaSituacaoNFCe($nota) {
        $result = $this->notaFiscal->consultaSituacaoNFe($nota);
        $xml = $result;
        
        $status = $this->getParameterXML($xml, 'cStat');
        $tProt = $this->getParameterXML($xml, 'nProt');
        $chNFe = $this->getParameterXML($xml, 'chNFe');
        
        $statusConsulta = 0;
        
        /* 100 - Autorizado o uso da NF-e
           101 - Cancelamento de NF-e homologado
           110 - Uso Denegado
           150 - Autorizado o uso da NF-e, autorização concedida fora do prazo 
           217 - NF-e não consta na base de dados da SEFAZ
           613 - Rejeicao: Codigo Numerico informado na Chave de Acesso difere do Codigo Numerico da NF-e
        */
            
        //if($chNFe === $nota['NRACESSONFCE'] && $nProt === $nota['NRPROTOCOLONFCE']){ 
           if (($status == '100') or ($status == '101') or ($status == '110') or ($status == '150')) {
                $dhRecbto = $this->getParameterXML($xml, 'dhRecbto');
                $dtHrProtocoNFCe = substr($dhRecbto, 0, 10).' '.substr($dhRecbto, 11, 8);
                $dtHrProtocoNFCe = date_create_from_format('Y-m-d H:s:i', $dtHrProtocoNFCe);
                $dtHrProtocoNFCe = $dtHrProtocoNFCe->format('d/m/Y H:s:i');
                
                if ($status == '100' or $status == '150') { 
                    $stStatusNFe = 'A';
                } else {
                    if ($status = '101')
                        $stStatusNFe = 'C';
                    if ($status = '110')
                        $stStatusNFe = 'D';
                }
                //verifica se o que esta aprovado é o que esta cadastrado
                if($chNFe !== $nota['NRACESSONFCE']){
                    //regera XML pq difere do que esta no banco
                    //define parametros 
                    $prt = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'CDLOJA', 'NRSEQVENDA'], $nota);
                    
                    $resposta = $this->transmissaoService->executarQuery(TransmissaoNfceService::ADQTRANSMISSAONFCE, $prt);
                    $resposta = $resposta->getResult()[0];
                    
                    $xmlRegerado =  $this->regerarXMLValido($chNFe, null, $nota, $xml);
                    if (!empty($xmlRegerado)) {
                    	
                    	$xmlAssinado = $this->notaFiscal->assinarXMLExterno($xmlRegerado, $resposta);
                    	if($xmlAssinado){
                    		$getQrCode = $this->getQrCode($xmlAssinado);
                    	}
                    	TransmissaoNfceController::escreverLog("Atualizando dados da venda.");
                    	$this->atualizarXMLBanco($xmlAssinado, $nota, $getQrCode);
                    }
                }
                
                $xMotivo = strlen($this->getParameterXML($xml, 'xMotivo')) > 3392 ? substr($this->getParameterXML($xml, 'xMotivo'), 0, 3393) : $this->getParameterXML($xml, 'xMotivo');
                
                //Copia-se 3933 caracteres do XML, pois a observação ocupa mais 67 caracteres
                $parametros['IDSTATUSNFCE'   ] = $stStatusNFe;
                $parametros['DSOBSSTATUSNFCE'] = $this->getParameterXML($xml, 'cStat') . ' - ' . $xMotivo . ' (Nota atualizada pelo processo de Consulta Situação de NFCe)';
                $parametros['NRPROTOCOLONFCE'] = $this->getParameterXML($xml, 'nProt');
                $parametros['DTHRPROTOCONFCE'] = $dtHrProtocoNFCe;
                $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
                $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
                $parametros['NRSEQVENDA'     ] = $nota['NRSEQVENDA'];
                $parametros['NRACESSONFCE'   ] = $chNFe;
                TransmissaoNfceController::escreverLog("Atualizando situacao atual - Nota Fiscal ".$nota['NRNOTAFISCALCE']." - através da chave de acesso. Status: ".$status);
                TransmissaoNfceController::escreverLog("");
                TransmissaoNfceController::escreverLog("Atualizando banco de dados ...");    
                $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDPROTOCNFCE, $parametros);
                $statusConsulta = 1;
                
                if (($nota['IDSITUVENDA'] == 'C') and ($stStatusNFe == 'A')) {
                    $venda = array(
                        'NRACESSONFCE'    => $chNFe,
                        'NRPROTOCOLONFCE' => $nota['NRPROTOCOLONFCE']
                    );
                    TransmissaoNfceController::escreverLog("Alterando informações quando venda está cancelada...");
                    $this->notaFiscal->cancelaVendaNFCE($nota['CDFILIAL'], $nota['CDCAIXA'], $nota['NRORG'], $nota['NRSEQVENDA'], $venda, $nota['DSRAZAOCANCNFCE'], $nota['CDOPERADOR']);
                    TransmissaoNfceController::escreverLog("Operação realizada.");
                }
                TransmissaoNfceController::escreverLog("");
                
           } else if ($status != '217') {
                //Copia-se 3933 caracteres do XML, pois a observação ocupa mais 67 caracteres
                $xMotivo = strlen($this->getParameterXML($xml, 'xMotivo')) > 3392 ? substr($this->getParameterXML($xml, 'xMotivo'), 0, 3393) : $this->getParameterXML($xml, 'xMotivo');
                $parametros['IDSTATUSNFCE'   ] = 'R';
                $parametros['DSOBSSTATUSNFCE'] = $this->getParameterXML($xml, 'cStat').' - ' . $xMotivo . ' (Nota atualizada pelo processo de Consulta Situação de NFCe)';
                $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
                $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
                $parametros['NRSEQVENDA'     ] = $nota['NRSEQVENDA'];  
                TransmissaoNfceController::escreverLog("Atualizando situacao atual - Nota Fiscal ".$nota['NRNOTAFISCALCE']." - através da chave de acesso. Status: ".$status);
                TransmissaoNfceController::escreverLog("");
                TransmissaoNfceController::escreverLog("Atualizando tabela...");
                $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDNFCE, $parametros);
           } 
           
           if(isset($parametros['DSOBSSTATUSNFCE'])){
               TransmissaoNfceController::escreverLog("Observação: ".$parametros['DSOBSSTATUSNFCE']);     
           }else{
                TransmissaoNfceController::escreverLog("");
           }
        return $statusConsulta;
    }
    
    public function inutilizarNotas($frontParametros) {
        $time_start = microtime(true);
        
        foreach($this->notasValidas as $nota) {
            TransmissaoNfceController::escreverLog("Nota Fiscal: ".$nota['NRNOTAFISCALCE']);
            $xml = $this->notaFiscal->invalidarNota($nota, $frontParametros);
            //$result = $this->notaFiscal->convertResult($xml);
            
            //TransmissaoNfceController::escreverLog($xml);
            $dhRecbto = $this->getParameterXML($xml, 'dhRecbto');
            $dtHrProtocoNFCe = substr($dhRecbto, 0, 10).' '.substr($dhRecbto, 11, 8);
            $dtHrProtocoNFCe = date_create_from_format('Y-m-d H:s:i', $dtHrProtocoNFCe);
            $dtHrProtocoNFCe = $dtHrProtocoNFCe->format('d/m/Y H:s:i');
            $CStat = $this->getParameterXML($xml, 'cStat');
            
            TransmissaoNfceController::escreverLog($CStat . ' - ' . $this->getParameterXML($xml, 'xMotivo'));
            $parametros = array();
            
            if ($CStat == '102') {
                $parametros['IDSTATUSNFCE'   ] = 'A';
                $parametros['DTHRINUTNFCE'   ] = $dtHrProtocoNFCe;
                $parametros['NRPROTOINUTNFCE'] = $this->getParameterXML($xml, 'NProt'); // RetInutNFe.DocumentElement.ChildNodes['infInut'].ChildNodes['NProt'].text; //RetInutNFe.DocumentElement.ChildNodes['NProt'].Text;
                $parametros['IDTPEMISINUNFCE'] = $frontParametros['IDTPEMISINUNFCE'];
                $parametros['DSRAZAOINUTNFCE'] = $frontParametros['DSOBSMOVBENECONS'];
                $parametros['DSOBSINUTNFCE'  ] = $this->getParameterXML($xml, 'xMotivo');
                $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
                $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
                $parametros['NRNOTAFISCALCE' ] = $nota['NRNOTAFISCALCE'];
                $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDINUTILIZANFCE, $parametros);
                
            } else {
                $parametros['IDSTATUSNFCE'   ] = 'R';
                $parametros['IDTPEMISINUNFCE'] = $frontParametros['IDTPEMISINUNFCE'];
                $parametros['DSRAZAOINUTNFCE'] = $frontParametros['DSOBSMOVBENECONS'];
                $parametros['DSOBSINUTNFCE'  ] = $this->getParameterXML($xml, 'xMotivo');
                $parametros['CDFILIAL'       ] = $nota['CDFILIAL'];
                $parametros['CDCAIXA'        ] = $nota['CDCAIXA'];
                $parametros['NRNOTAFISCALCE' ] = $nota['NRNOTAFISCALCE'];
                $this->transmissaoService->runQuery(TransmissaoNfceService::ACMUPDINUTILIZA, $parametros);
            }
            TransmissaoNfceController::escreverLog("");
        }
        
        $time_end = microtime(true);
        $time = $time_end - $time_start;
        TransmissaoNfceController::escreverLog("Tempo gasto: ".number_format($time, 4, '.', '')."s");
        
    }
    
    private function validadeChaveAcesso($chave){
        if (ctype_digit($chave) && strlen($chave) == 44 ){
            return true;
        }else{
            return false;
        }
    }
    
    private function xmlToArray($xml, $options = array()) {
        $defaults = array(
            'namespaceSeparator' => ':', // você pode querer que isso seja algo diferente de um cólon
            'attributePrefix' => '@',    // para distinguir entre os nós e os atributos com o mesmo nome
            'alwaysArray' => array(),    // array de tags que devem sempre ser array
            'autoArray' => true,         // só criar arrays para as tags que aparecem mais de uma vez
            'textContent' => '$',        // chave utilizada para o conteúdo do texto de elementos
            'autoText' => true,          // pular chave "textContent" se o nó não tem atributos ou nós filho
            'keySearch' => false,        // pesquisa opcional e substituir na tag e nomes de atributos
            'keyReplace' => false        // substituir valores por valores acima de busca
        );
        $options = array_merge($defaults, $options);
        $namespaces = $xml->getDocNamespaces();
        $namespaces[''] = null; // adiciona namespace base(vazio) 
    
        // Obtém os atributos de todos os namespaces
        $attributesArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->attributes($namespace) as $attributeName => $attribute) {
                // Substituir caracteres no nome do atributo
                if ($options['keySearch']) $attributeName =
                        str_replace($options['keySearch'], $options['keyReplace'], $attributeName);
                $attributeKey = $options['attributePrefix']
                        . ($prefix ? $prefix . $options['namespaceSeparator'] : '')
                        . $attributeName;
                $attributesArray[$attributeKey] = (string)$attribute;
            }
        }
    
        // Obtém nós filhos de todos os namespaces
        $tagsArray = array();
        foreach ($namespaces as $prefix => $namespace) {
            foreach ($xml->children($namespace) as $childXml) {
                // Recursividade em nós filho
                $childArray = $this->xmlToArray($childXml, $options);
                list($childTagName, $childProperties) = $this->myEach($childArray);
    
                // Substituir caracteres no nome da tag
                if ($options['keySearch']) $childTagName =
                        str_replace($options['keySearch'], $options['keyReplace'], $childTagName);
                // Adiciona um prefixo namespace, se houver
                if ($prefix) $childTagName = $prefix . $options['namespaceSeparator'] . $childTagName;
    
                if (!isset($tagsArray[$childTagName])) {
                    // Só entra com esta chave
                    // Testa se as tags deste tipo deve ser sempre matrizes, não importa a contagem de elementos
                    $tagsArray[$childTagName] =
                            in_array($childTagName, $options['alwaysArray']) || !$options['autoArray']
                            ? array($childProperties) : $childProperties;
                } elseif (
                    is_array($tagsArray[$childTagName]) && array_keys($tagsArray[$childTagName]) 
                    === range(0, count($tagsArray[$childTagName]) - 1)
                ) {
                    $tagsArray[$childTagName][] = $childProperties;
                } else {
                    $tagsArray[$childTagName] = array($tagsArray[$childTagName], $childProperties);
                }
            }
        }
    
        // Obtém o texto do nó
        $textContentArray = array();
        $plainText = trim((string)$xml);
        if ($plainText !== '') $textContentArray[$options['textContent']] = $plainText;
    
        $propertiesArray = !$options['autoText'] || $attributesArray || $tagsArray || ($plainText === '')
                ? array_merge($attributesArray, $tagsArray, $textContentArray) : $plainText;
    
        // Retorna o nó como array
        return array(
            $xml->getName() => $propertiesArray
        );
    }
    
    /**
     * @param array $array the array to be converted
     * @param string? $rootElement if specified will be taken as root element, otherwise defaults to 
     *                <root>
     * @param SimpleXMLElement? if specified content will be appended, used for recursion
     * @return string XML version of $array
     */
    private function arrayToXml($array) {
      $xmlRetorno = simplexml_load_string(self::DEFAULT_XML);
      foreach($array as $key => $value){
          $infNFe = $xmlRetorno->addChild('infNFe');
          $infNFe->addAttribute('Id', $value['@Id']);
          $infNFe->addAttribute('versao', $value['@versao']);
          
          //forma o destinatario
          $ide = $infNFe->addChild('ide');
          foreach($value['ide'] as $k => $v){
              $ide->addChild($k, $v);
          }
          
          //forma o emitente
          $emit = $infNFe->addChild('emit');
          foreach($value['emit'] as $k => $v){
              if(is_array($v)){
                  $enderEmit = $emit->addChild('enderEmit');
                  foreach($v as $ke => $vl){
                      $enderEmit->addChild($ke, $vl);
                  }
              }else{
                  $emit->addChild($k, $v);
              }
          }
          
          if(!isset($value['det']['@nItem'])){
              foreach($value['det'] as $k => $v){
                  $det = $infNFe->addChild('det');
                  $det->addAttribute('nItem', $v['@nItem']);
                  $prod = $det->addChild('prod');
                  foreach($v['prod'] as $pe => $pl){
                      if(is_array($pl)){
                          $prd = $prod->addChild($pe);
                          foreach($pl as $im => $il){
                              if(is_array($il)){
                                  $pr = $prd->addChild($im);
                                  foreach($il as $chil => $viml){
                                      $pr->addChild($chil, $viml);
                                  }
                              }else{
                                  $prd->addChild($im, $il);
                              }
                          }
                      }else{
                          $prod->addChild($pe, $pl);
                      }
                  }
                  
                  //formara os impostos 
                  $imposto = $det->addChild('imposto');
                  foreach($v['imposto'] as $pe => $pl){
                      if(is_array($pl)){
                          $impl = $imposto->addChild($pe);
                          foreach($pl as $im => $il){
                              if(is_array($il)){
                                  $im = $impl->addChild($im);
                                  foreach($il as $chil => $viml){
                                      $im->addChild($chil, $viml);
                                  }
                              }else{
                                  $impl->addChild($im, $il);
                              }
                          }
                      }else{
                          $imposto->addChild($pe, $pl);
                      }
                  }
              }
          }else{
              $det = $infNFe->addChild('det');
              $det->addAttribute('nItem', $value['det']['@nItem']);
              $prod = $det->addChild('prod');
              foreach($value['det']['prod'] as $pe => $pl){
                  if(is_array($pl)){
                      $prd = $prod->addChild($pe);
                      foreach($pl as $im => $il){
                          if(is_array($il)){
                              $pr = $prd->addChild($im);
                              foreach($il as $chil => $viml){
                                  $pr->addChild($chil, $viml);
                              }
                          }else{
                              $prd->addChild($im, $il);
                          }
                      }
                  }else{
                      $prod->addChild($pe, $pl);
                  }
              }
              
              //formara os impostos 
              $imposto = $det->addChild('imposto');
              foreach($value['det']['imposto'] as $pe => $pl){
                  if(is_array($pl)){
                      $impl = $imposto->addChild($pe);
                      foreach($pl as $im => $il){
                          if(is_array($il)){
                              $im = $impl->addChild($im);
                              foreach($il as $chil => $viml){
                                  $im->addChild($chil, $viml);
                              }
                          }else{
                              $impl->addChild($im, $il);
                          }
                      }
                  }else{
                      $imposto->addChild($pe, $pl);
                  }
              }
          }
          
          //formara o total
          $total = $infNFe->addChild('total');
          foreach($value['total'] as $k => $v){
              if(is_array($v)){
                  $ICMSTot = $total->addChild('ICMSTot');
                  foreach($v as $ke => $vl){
                      $ICMSTot->addChild($ke, $vl);
                  }
              }else{
                  $total->addChild($k, $v);
              }
          }
          
          //formara transportadora
          $transp = $infNFe->addChild('transp');
          foreach($value['transp'] as $k => $v){
              $transp->addChild($k, $v);
          }
          
          //formara pagamento
          $pag = $infNFe->addChild('pag');
          foreach($value['pag'] as $k => $v){
              if(is_array($v)){
                  $detPag = $pag->addChild('detPag');
                  foreach($v as $ke => $vl){
                      if (is_array($vl)){
                          $card = $detPag->addChild('card');
                         foreach($vl as $c => $cl){
                             $card->addChild($c, $cl);
                         }  
                      } else{
                          $detPag->addChild($ke, $vl);
                      }
                  }
              }else{
                  $pag->addChild($k, $v);
              }
          }
          
          //formara informacoes adicionais
          $infAdic = $infNFe->addChild('infAdic');
          foreach($value['infAdic'] as $k => $v){
              if(is_array($v)){
                  $obsCont = $infAdic->addChild('obsCont');
                  foreach($v as $ke => $vl){
                      if($ke === '@xCampo'){
                        $obsCont->addAttribute('xCampo', $vl);   
                      }else{
                        $obsCont->addChild($ke, $vl);    
                      }
                  }
              }else{
                  $infAdic->addChild($k, $v);
              }
          }
          
          //formara informacoes do responsavel tecnico caso tenha
          if(!empty($array['infNFe']['infRespTec'])){
            $infRespTec = $infNFe->addChild('infRespTec');
            foreach($value['infRespTec'] as $k => $v){
              $infRespTec->addChild($k, $v);
            }   
          }
      }
      return $xmlRetorno->asXML();
    }
    
    private function myEach(&$arr) {
        $key = key($arr);
        $result = ($key === null) ? false : [$key, current($arr), 'key' => $key, 'value' => current($arr)];
        next($arr);
        return $result;
    }
    
    const DEFAULT_XML = '
        <NFe xmlns="http://www.portalfiscal.inf.br/nfe">
            
        </NFe>';
    
    
} 
